package com.malk.huaxiaojinshu.utils;


import com.alibaba.fastjson.JSON;
import com.malk.utils.UtilHttp;

import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.URL;
import java.util.Base64;
import java.util.HashMap;
import java.util.Map;

public class XiaomanAuthorization {


    /**
     * 获取token
     * @param clientId
     * @param clientSecret
     * @param username
     * @param password
     * @return
     */
    public static String getToken(String clientId, String clientSecret, String username, String password) {

        String authString = clientId+":"+clientSecret;
        String encodedAuthString = Base64.getEncoder().encodeToString(authString.getBytes());
        System.out.println(encodedAuthString);
        Map<String, String> header = new HashMap<>();
        header.put("Authorization", "Basic "+encodedAuthString);


        Map<String, String> body = new HashMap<>();
        body.put("client_id", clientId);
        body.put("client_secret", clientSecret);
        body.put("grant_type", "password");
        body.put("scope", "company opportunity user");
        body.put("username", username);
        body.put("password", password);
        String str = UtilHttp.doPost("https://api-sandbox.xiaoman.cn/v1/oauth2/access_token", header, null, body);
        String token = String.valueOf(((Map) JSON.parse(str)).get("access_token"));
        return token;
    }


    /**
     * 获取私海客户分组信息
     * @param token
     * @return
     */
    public static String getGroupId(String token) {

        String url = "https://api-sandbox.xiaoman.cn/v1/company/fields/selector";
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer "+token);

        Map<String, String> params = new HashMap<>();
        params.put("field","group_id");
        String rsp = UtilHttp.doGet(url, headers, params);
        return rsp;
    }



    /**
     * 获取公海客户分组信息
     * @param token
     * @return
     */
    public static String getPoolGroupId(String token) {

        String url = "https://api-sandbox.xiaoman.cn/v1/company/fields/selector";
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer "+token);

        Map<String, String> params = new HashMap<>();
        params.put("field","pool_id");
        String rsp = UtilHttp.doGet(url, headers, params);
        return rsp;
    }


    /**
     *
     * @param token
     * @param groupId
     * @param page         页码
     * @param pageSize   每页数量
     * @param status   1所有客户  0私海客户
     * @return
     */

    public static String getCompany(String token,String groupId,int page,int pageSize,int status) {

        String url = "https://api-sandbox.xiaoman.cn/v1/company/list";
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer "+token);

        Map<String, Object> params = new HashMap<>();
        params.put("start_index",page);
        params.put("count",pageSize);
        params.put("removed", 0);
        params.put("all", status);
        params.put("group_id", groupId);

        String rsp = UtilHttp.doGet(url, headers, params);
        return rsp;
    }


    /**
     * 获取客户信息
     * @param token
     * @return
     */
    public static String getCompanyInfo(String token,String companyId) {

        String url = "https://api-sandbox.xiaoman.cn/v1/company/info";
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer "+token);

        Map<String, Object> params = new HashMap<>();
        params.put("company_id", companyId);
        String rsp = UtilHttp.doGet(url, headers, params);
        return rsp;
    }


    /**
     * 获取商机信息
     * @param token
     * @param statrTime   “yyyy-MM-dd”
     * @param endTime    “yyyy-MM-dd”
     * @return
     */
    public static String getOpportunity(String token,int page,int pageSize,String statrTime,String endTime) {


        String url = "https://api-sandbox.xiaoman.cn/v1/opportunity/list";
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer "+token);

        Map<String, Object> params = new HashMap<>();
        params.put("start_index", page);
        params.put("count", pageSize);
        params.put("start_time", statrTime);
        params.put("end_time", endTime);
        String rsp = UtilHttp.doGet(url, headers, params);

        return rsp;
    }


    /**
     * 获用户信息
     */
    public static String getUserInfo(String token,String userId) {


        String url = "https://api-sandbox.xiaoman.cn/v1/user/info";
        Map<String, String> headers = new HashMap<>();
        headers.put("Authorization", "Bearer "+token);

        Map<String, Object> params = new HashMap<>();
        params.put("user_id",userId);
        String rsp = UtilHttp.doGet(url, headers, params);

        return rsp;
    }

}
