package com.malk.yibaoju.schedule;

import com.malk.yibaoju.service.YBJService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.autoconfigure.condition.ConditionalOnProperty;
import org.springframework.context.annotation.Configuration;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;

import java.time.LocalDate;

/**
 * @EnableScheduling 开启定时任务 [配置参考McScheduleTask]
 */
@Slf4j
@Configuration
@EnableScheduling
@ConditionalOnProperty(name = {"spel.scheduling"})
public class ScheduleTask {

    @Autowired
    private YBJService ybjService;

    /**
     * 每月25日0点30分执行定时任务
     */
//    @Scheduled(cron = "0 30 0 25 * ?")
//    public void syncDingTalkFailedList() {
//        try {
//            ybjService.syncCreatePlan();
//        } catch (Exception e) {
//            // 记录错误信息
//            e.printStackTrace();
//        }
//    }

    /**
     * 每天凌晨00点30分更新计划护理的任务状态
     */
    @Scheduled(cron = " 0 20 0 * * ? ")
    public void syncDingTalkStatus() {
        try {
            ybjService.syncUpdatePlanStatus();
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }
/**-------------------------------------重构------------------------------------------**/


    /**
     * 每月27日05点30分执行定时任务生成数据库下月的护理计划
     */
    @Scheduled(cron = "0 30 0 25 * ?")
    public void insertDatabaseTask() {
        try {
            ybjService.insertDatabaseTask();
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }

    /**
     * 每5分执行定时任务同步数据库护理计划到宜搭
     */
    @Scheduled(cron = "0 0/5 * * * ?")
    public void databaseToYiDaTask() {
        try {
            ybjService.databaseToYiDaTask();
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }

    /**
     * 6点到23点每3分执行定时任务同步宜搭到数据库更新是否已同步字段和流程id
     */
    @Scheduled(cron = "0 0/3 4-23 * * ?")
    public void afterDatabaseToYiDaTask() {
        try {
            ybjService.afterDatabaseToYiDaTask();
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }
    /**
     * 每天晚上八点执行定时任务删除数据库逻辑删除的数据及宜搭
     */
    @Scheduled(cron = "0 0 6/1 * * ?")
    public void deleteBothTask() {
        try {
            ybjService.deleteBothInfo();
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }
    /**
     * 每天凌晨00点05分更新计划护理的任务状态
     */
    @Scheduled(cron = " 0 5 0 * * ? ")
    public void updateDBStatus() {
        log.info("开始执行更新护理计划状态任务");
        try {
            ybjService.updateDBStatus();
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }

    /**
     * 每天晚上23点00分删除前30天打卡数据定时任务
     */
    @Scheduled(cron = " 0 00 23 * * ? ")
    public void deleteClockIn() {
        log.info("开始执行删除前30天打卡数据定时任务");
        try {
            ybjService.deleteClockIn();
        } catch (Exception e) {
            // 记录错误信息
            e.printStackTrace();
        }
    }
}