package com.malk.yibaoju.service.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.ArrayUtil;
import cn.hutool.core.util.ObjectUtil;
import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.malk.server.aliwork.YDConf;
import com.malk.server.aliwork.YDParam;
import com.malk.service.aliwork.YDClient;
import com.malk.service.aliwork.YDService;
import com.malk.utils.UtilDateTime;
import com.malk.utils.UtilExcel;
import com.malk.utils.UtilMap;
import com.malk.yibaoju.entity.Plan;
import com.malk.yibaoju.mapper.PlanMapper;
import com.malk.yibaoju.service.PlanService;
import com.malk.yibaoju.service.YBJService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.file.Files;
import java.nio.file.Paths;
import java.sql.*;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.time.LocalDate;
import java.time.LocalDateTime;
import java.time.format.TextStyle;
import java.time.temporal.TemporalAdjusters;
import java.util.Date;
import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
public class YBJServiceImpl implements YBJService {


    @Autowired
    private YDClient ydClient;

    @Autowired
    private YDService ydService;

    @Autowired
    private PlanMapper planMapper;

    @Value("${spring.datasource.username}")
    private String username;

    @Value("${spring.datasource.password}")
    private String password;

    @Value("${spring.datasource.url}")
    private String url;

    @Value("${spring.datasource.driver-class-name}")
    private String driver;

    @Autowired
    private PlanService planService;

    SimpleDateFormat dateFormat = new SimpleDateFormat("yyyy-MM-dd HH:mm:ss");

    LocalDate currentDate = LocalDate.now();
    //获取下月1号的日期
    LocalDate firstDayOfNextMonth = currentDate.plusMonths(1).with(TemporalAdjusters.firstDayOfMonth());
    // 数据库下月数据日期
    String nextDataMonth = firstDayOfNextMonth.getYear() +"-"+ String.format("%02d",firstDayOfNextMonth.getMonthValue());
    // 数据库当月数据日期
    String dataMonth = currentDate.getYear() +"-"+ String.format("%02d", currentDate.getMonthValue());
    //获取下月月末最后一天日期
    LocalDate lastDayOfNextMonth = currentDate.plusMonths(1).with(TemporalAdjusters.lastDayOfMonth());



    /**
     * 每月创建护理计划
     */
    @Override
    @SneakyThrows
    public void syncNursePlan() {

        YDParam ydParam = new YDParam();
        log.info("创建护理计划定时任务Start", new Date());

        ydParam = YDParam.builder()
                .formUuid("FORM-RK966E7105DFD27FA7EQHAOH9IFS2RN6MP3OL2")  //获取护理计划
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lo2b6bvf", "正常")))//护理对象暂停和正常、月度计划创建只查询正常的护理计划
                .build();
        //pagesize设为1获取总数
        ydParam.setPageSize(1);
        long totalCount = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
        float pageSize = 50;
        ydParam.setCurrentPage(1);
        ydParam.setPageSize((int) pageSize);

        log.info("totalCount:----{}----", totalCount);

        List<Map> dataList = new ArrayList<>();
        for (int page = 1; page <= Math.ceil(totalCount / pageSize); page++) {
            ydParam.setCurrentPage(page);
            //创建护理计划  1次获取50条
            dataList = (List<Map>) ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getData();

            log.info("dataList:{}-----page:" + page + "---", dataList.size());

            LocalDate currentDate = LocalDate.now();
            //获取当前日期是本月几号
            int dayOfMonth = currentDate.getDayOfMonth();
            //获取本月天数
            int daysInMonth = currentDate.lengthOfMonth();

            dataList.forEach(dataItem -> {

                Map formMap = (Map) dataItem.get("formData");

                List<Map> mapList = (List<Map>) formMap.get("tableField_lo3pmohx");

                for (int i = 0; i <= (daysInMonth - dayOfMonth); i++) {
                    //获取当前日期是本月几号
                    LocalDate currentDay = LocalDate.now();
                    //获取是周几
                    String dayOfWeek = currentDay.plusDays(i).getDayOfWeek().getDisplayName(TextStyle.FULL, new Locale("zh", "CN"));
                    //获取循环每天的日期
                    LocalDate nextDay = currentDate.plusDays(i);
                    List<Map> maps = mapList.stream().filter(items -> items.get("selectField_lo3pmohy").equals(dayOfWeek.replace("星期", "周"))).collect(Collectors.toList());

                    for (Map map : maps) {
                        Map hashMap = new HashMap();

                        hashMap.put("associationFormField_lo3pmohz", JSON.parse(map.get("associationFormField_lo3pmohz_id").toString()));  //护理人员名称
                        hashMap.put("textField_lo3pmoi0", map.get("textField_lo3pmoi0"));  //护理人员所属机构
                        hashMap.put("textField_lombve2g", map.get("textField_lomchzi3"));  //护理人员身份证
                        hashMap.put("textField_lo3pmoi3", map.get("textField_lo3pmoi3"));  //护理人员手机号
                        hashMap.put("associationFormField_lo3pmohw", JSON.parse(formMap.get("associationFormField_lo3pmohw_id").toString()));  //护理对象姓名
                        hashMap.put("textField_lnyhv5tn", formMap.get("textField_lnyhv5tn")); //护理对象身份证
                        hashMap.put("selectField_lnyhv5tp", formMap.get("selectField_lnyhv5tp"));  //护理对象等级
                        hashMap.put("textField_lnyhv5tr", formMap.get("textField_lnyhv5tr"));  //护理对象住址
                        hashMap.put("selectField_lo3pmohy", dayOfWeek.replace("星期", "周"));  //护理时间  （星期几）
                        hashMap.put("dateField_lred6eoj", UtilDateTime.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3"), "yyyy-MM-dd HH:mm")); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                        hashMap.put("dateField_lred6eok", UtilDateTime.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4"), "yyyy-MM-dd HH:mm")); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                        hashMap.put("selectField_lqcgsbw2", "未执行");  //执行情况
                        hashMap.put("selectField_lo56u5fn", "待上报");  //任务状态

                        hashMap.put("textField_lvae7q5n", formMap.get("textField_lvae99fj")); //护理对象经度
                        hashMap.put("textField_lvae7q5o", formMap.get("textField_lvae99fk"));  //护理对象纬度


                        hashMap.put("textField_lrx0r38x", String.valueOf(nextDay));  //护理日期-文本
                        hashMap.put("selectField_lr36z6hd", "是");  //任务状态
                        hashMap.put("textField_lo55rupj", formMap.get("textField_lo55rupj"));  //护理对象姓名-文本
                        hashMap.put("textField_lo3pmoi4", map.get("textField_lo3pmoi4"));  //护理证书类型
                        hashMap.put("employeeField_lo3pmoi6", JSON.parse(map.get("employeeField_lo3pmoi6_id").toString()));  //护理人员姓名-成员
                        hashMap.put("textField_lpgi8scj", map.get("textField_lo701pwy"));  //计划明细id
                        hashMap.put("textField_lq9llmhl", formMap.get("textField_lop4v4qx"));  //护理对象ID

                        if (formMap.get("departmentSelectField_lxztlnnx_id") != null) {
                            hashMap.put("departmentSelectField_lxzto1e1", String.valueOf(formMap.get("departmentSelectField_lxztlnnx_id")));  //部门
                        }
                        try {
                            Thread.sleep(200);
                        } catch (InterruptedException e) {
                            throw new RuntimeException(e);
                        }
                        try {
                            ydClient.operateData(YDParam.builder()
                                    .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")
                                    .searchCondition(JSON.toJSONString(UtilMap.map("dateField_lred6eoj, textField_lnyhv5tn, textField_lombve2g", Arrays.asList(UtilDateTime.parse(nextDay + " 00:00:01", "yyyy-MM-dd HH:mm:ss"), UtilDateTime.parse(nextDay + " 23:59:59", "yyyy-MM-dd HH:mm:ss")), formMap.get("textField_lnyhv5tn"), map.get("textField_lomchzi3"))))
                                    .formDataJson(JSON.toJSONString(hashMap))
                                    .build(), YDConf.FORM_OPERATION.upsert);
                        } catch (Exception e) {
                            log.info("异常数据{}", hashMap);
                        }

                    }

                }
            });
        }

        log.info("创建护理计划定时任务end", new Date());
    }


    /**
     * 定时任务每月25号创建下月的数据
     */
    @Override
    @SneakyThrows
    public void syncCreatePlan() {

        YDParam ydParam = new YDParam();
        log.info("创建下月护理计划定时任务Start:{}", new Date());

        ydParam = YDParam.builder()
                .formUuid("FORM-RK966E7105DFD27FA7EQHAOH9IFS2RN6MP3OL2")  //获取护理计划
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lo2b6bvf", "正常")))//护理对象暂停和正常、月度计划创建只查询正常的护理计划
                .build();
        //pagesize设为1获取总数
        ydParam.setPageSize(1);
        long totalCount = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
        float pageSize = 50;
        ydParam.setCurrentPage(1);
        ydParam.setPageSize((int) pageSize);
        List<Map> dataList = new ArrayList<>();

        LocalDate currentDate = LocalDate.now();
        //获取下月1号的日期
        LocalDate firstDayOfNextMonth = currentDate.plusMonths(1).with(TemporalAdjusters.firstDayOfMonth());
        //获取日期是几号
        int dayOfMonth = firstDayOfNextMonth.getDayOfMonth();
        //获取月天数
        int daysInMonth = firstDayOfNextMonth.lengthOfMonth();

        for (int page = 1; page <= Math.ceil(totalCount / pageSize); page++) {
            ydParam.setCurrentPage(page);
            //创建护理计划  1次获取50条
            dataList = (List<Map>) ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getData();
            log.info("dataList:{}-----page:" + page + "---", dataList.size());

            dataList.forEach(dataItem -> {
                Map formMap = (Map) dataItem.get("formData");
                List<Map> mapList = (List<Map>) formMap.get("tableField_lo3pmohx");
                for (int i = 0; i <= (daysInMonth - dayOfMonth); i++) {
                    //获取是周几
                    String dayOfWeek = firstDayOfNextMonth.plusDays(i).getDayOfWeek().getDisplayName(TextStyle.FULL, new Locale("zh", "CN"));
                    //获取循环每天的日期
                    LocalDate nextDay = firstDayOfNextMonth.plusDays(i);
                    List<Map> maps = mapList.stream().filter(items -> items.get("selectField_lo3pmohy").equals(dayOfWeek.replace("星期", "周"))).collect(Collectors.toList());
                    for (Map map : maps) {
                        Map hashMap = new HashMap();

                        hashMap.put("associationFormField_lo3pmohz", JSON.parse(map.get("associationFormField_lo3pmohz_id").toString()));  //护理人员名称
                        hashMap.put("textField_lo3pmoi0", map.get("textField_lo3pmoi0"));  //护理人员所属机构
                        hashMap.put("textField_lombve2g", map.get("textField_lomchzi3"));  //护理人员身份证
                        hashMap.put("textField_lo3pmoi3", map.get("textField_lo3pmoi3"));  //护理人员手机号
                        hashMap.put("associationFormField_lo3pmohw", JSON.parse(formMap.get("associationFormField_lo3pmohw_id").toString()));  //护理对象姓名
                        hashMap.put("textField_lnyhv5tn", formMap.get("textField_lnyhv5tn")); //护理对象身份证
                        hashMap.put("selectField_lnyhv5tp", formMap.get("selectField_lnyhv5tp"));  //护理对象等级
                        hashMap.put("textField_lnyhv5tr", formMap.get("textField_lnyhv5tr"));  //护理对象住址
                        hashMap.put("selectField_lo3pmohy", dayOfWeek.replace("星期", "周"));  //护理时间  （星期几）
                        hashMap.put("dateField_lred6eoj", UtilDateTime.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3"), "yyyy-MM-dd HH:mm")); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                        hashMap.put("dateField_lred6eok", UtilDateTime.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4"), "yyyy-MM-dd HH:mm")); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                        hashMap.put("selectField_lqcgsbw2", "未执行");  //执行情况
                        hashMap.put("selectField_lo56u5fn", "待上报");  //任务状态

                        hashMap.put("textField_lvae7q5n", formMap.get("textField_lvae99fj")); //护理对象经度
                        hashMap.put("textField_lvae7q5o", formMap.get("textField_lvae99fk"));  //护理对象纬度

                        hashMap.put("textField_lrx0r38x", String.valueOf(nextDay));  //护理日期-文本
                        hashMap.put("selectField_lr36z6hd", "是");  //任务状态
                        hashMap.put("textField_lo55rupj", formMap.get("textField_lo55rupj"));  //护理对象姓名-文本
                        hashMap.put("textField_lo3pmoi4", map.get("textField_lo3pmoi4"));  //护理证书类型
                        hashMap.put("employeeField_lo3pmoi6", JSON.parse(map.get("employeeField_lo3pmoi6_id").toString()));  //护理人员姓名-成员
                        hashMap.put("textField_lpgi8scj", map.get("textField_lo701pwy"));  //计划明细id
                        hashMap.put("textField_lq9llmhl", formMap.get("textField_lop4v4qx"));  //护理对象ID
                        if (formMap.get("departmentSelectField_lxztlnnx_id") != null) {
                            hashMap.put("departmentSelectField_lxzto1e1", String.valueOf(formMap.get("departmentSelectField_lxztlnnx_id")));  //部门
                        }
                        try {
                            Thread.sleep(200);
                        } catch (InterruptedException e) {
                            log.info("睡眠异常");
                        }
                        try {
                            ydClient.operateData(YDParam.builder()
                                    .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")
//                                    .searchCondition(JSON.toJSONString(UtilMap.map("dateField_lred6eoj, textField_lnyhv5tn, textField_lombve2g",Arrays.asList(UtilDateTime.parse( nextDay+ " 00:00:01","yyyy-MM-dd HH:mm:ss"),UtilDateTime.parse( nextDay+ " 23:59:59","yyyy-MM-dd HH:mm:ss")),formMap.get("textField_lnyhv5tn"),map.get("textField_lomchzi3"))))
                                    .formDataJson(JSON.toJSONString(hashMap))
                                    .build(), YDConf.FORM_OPERATION.create);
                        } catch (Exception e) {
                            log.info("异常数据{}", hashMap);
                        }
                    }
                }
            });
        }
        log.info("创建下月护理计划定时任务end", new Date());
    }

    /**
     * 定时任务创建下月护理计划插入数据库
     */
    @Override
    public void insertDatabaseTask() {
        log.info("创建下月护理计划定时任务生成到数据库:{}", new Date());
        YDParam ydParam = new YDParam();

        ydParam = YDParam.builder()
                .formUuid("FORM-RK966E7105DFD27FA7EQHAOH9IFS2RN6MP3OL2")  //获取护理计划
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lo2b6bvf", "正常")))//护理对象暂停和正常、月度计划创建只查询正常的护理计划
                .build();
        //获取日期是几号
        int dayOfMonth = firstDayOfNextMonth.getDayOfMonth();
        //获取月天数
        int daysInMonth = firstDayOfNextMonth.lengthOfMonth();
        try {
            Class.forName(driver);
            Connection connection = DriverManager.getConnection(url, username, password);
            final PreparedStatement[] ps = {null};
            String insertSql = "INSERT INTO `yida_plan_details`( `caregiver_form`, `caregiver_name`, `caregiver_member`, `caregiver_department`, " +
                    "`caregiver_card`, `caregiver_phone`, `department`, `care_object_form`, `care_object_name`, `care_object_card`, `care_object_level`, " +
                    "`care_object_address`, `care_object_longitude`, `care_object_latitude`, `within_the_week` , `care_start_time`, `care_end_time`, " +
                    "`execute_status`, `task_status`, `plan_id`, `care_object_id`, `certificate_type`, `is_choose`, `care_date_text` , `start_time_text` ," +
                    " `end_time_text` ,`form_inst_id`, `data_month`, `is_cogradient`, `start_time`, `end_time`, `is_delete`) " +
                    "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) ";
            ps[0] = connection.prepareStatement(insertSql);
//            for (int page = 1; page <= Math.ceil(totalCount / pageSize); page++) {
//                ydParam.setCurrentPage(page);
            //创建护理计划  1次获取50条
//                dataList = (List<Map>) ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getData();
            List<Map> dataList = ydService.queryFormData_all(ydParam);
//                log.info("dataList:{}-----page:" + page + "---", dataList.size());

            dataList.forEach(dataItem -> {
//                    Map formMap = (Map) dataItem.get("formData");
                List<Map> mapList = (List<Map>) dataItem.get("tableField_lo3pmohx");
                try {
                    for (int i = 0; i <= (daysInMonth - dayOfMonth); i++) {
                        //获取是周几
                        String dayOfWeek = firstDayOfNextMonth.plusDays(i).getDayOfWeek().getDisplayName(TextStyle.FULL, new Locale("zh", "CN"));
                        //获取循环每天的日期
                        LocalDate nextDay = firstDayOfNextMonth.plusDays(i);
                        List<Map> maps = mapList.stream().filter(items -> items.get("selectField_lo3pmohy").equals(dayOfWeek.replace("星期", "周"))).collect(Collectors.toList());

                        for (Map map : maps) {
                            Object parse = JSON.parse(map.get("associationFormField_lo3pmohz_id").toString());
                            JSONArray jsonArray = JSON.parseArray(parse.toString());
                            JSONObject jsonObject = (JSONObject) jsonArray.get(0);
                            // 创建Statement对象，用于执行SQL语句
                            ps[0].setObject(1,map.get("associationFormField_lo3pmohz_id").toString());  //护理人员名称(关联表单)
                            ps[0].setObject(2, jsonObject.get("title"));  //护理人员名称
                            ps[0].setObject(3,map.get("employeeField_lo3pmoi6_id").toString());  //护理人员姓名-成员
                            ps[0].setObject(4,map.get("textField_lo3pmoi0"));  //护理人员所属机构
                            ps[0].setObject(5,map.get("textField_lomchzi3"));  //护理人员身份证
                            ps[0].setObject(6,map.get("textField_lo3pmoi3"));  //护理人员手机号
                            if (dataItem.get("departmentSelectField_lxztlnnx_id") != null) {
                                ps[0].setObject(7,String.valueOf(dataItem.get("departmentSelectField_lxztlnnx_id")));  //部门
                            }else {
                                ps[0].setObject(7,null);  //部门
                            }
                            ps[0].setObject(8,dataItem.get("associationFormField_lo3pmohw_id").toString());//护理对象姓名(关联表单)
                            ps[0].setObject(9,dataItem.get("textField_lo55rupj"));  //护理对象姓名-文本
                            ps[0].setObject(10,dataItem.get("textField_lnyhv5tn")); //护理对象身份证
                            ps[0].setObject(11,dataItem.get("selectField_lnyhv5tp"));  //护理对象等级
                            ps[0].setObject(12,dataItem.get("textField_lnyhv5tr"));  //护理对象住址
                            ps[0].setObject(13, dataItem.get("textField_lvae99fj")); //护理对象经度
                            ps[0].setObject(14, dataItem.get("textField_lvae99fk"));  //护理对象纬度
                            ps[0].setObject(15, dayOfWeek.replace("星期", "周"));  //护理时间  （星期几）
                            // 将字符串时间转换为Date对象
                            Date startDate = dateFormat.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3") + ":00");
                            Date endDate = dateFormat.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4") + ":00");
                            ps[0].setObject(16, startDate.getTime()); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                            ps[0].setObject(17, endDate.getTime()); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                            ps[0].setObject(18, "未执行");  //执行情况
                            ps[0].setObject(19, "待上报");  //任务状态
                            ps[0].setObject(20, map.get("textField_lo701pwy"));  //计划明细id
                            ps[0].setObject(21, dataItem.get("textField_lop4v4qx"));  //护理对象ID
                            ps[0].setObject(22, map.get("textField_lo3pmoi4"));  //护理证书类型
                            ps[0].setObject(23, "是");  //是否可选
                            ps[0].setObject(24, String.valueOf(nextDay));  //护理日期-文本
                            ps[0].setObject(25, String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3")); //护理开始时间。 用当前日期+护理计划的  时分
                            ps[0].setObject(26, String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4")); //护理结束时间。 用当前日期+护理计划的  时分
                            ps[0].setObject(27, dataItem.get("formInstanceId").toString()); //护理结束时间。 用当前日期+护理计划的  时分
                            ps[0].setObject(28, nextDataMonth); //下月数据日期
                            ps[0].setObject(29, "否"); //是否已同步宜搭
                            ps[0].setObject(30,map.get("selectField_lr1uxpu3"));
                            ps[0].setObject(31,map.get("selectField_lr1uxpu4"));
                            ps[0].setObject(32,0);
                            // 执行SQL查询
                            ps[0].addBatch();
                        }
                    }
                    ps[0].executeBatch();
                    ps[0].clearBatch();
                } catch (SQLException e) {
                    log.info("sql或插入数据库执行失败:{}",dataItem.get("formInstanceId"));
                    e.printStackTrace();
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }
            });
//            }
            // 关闭资源
            ps[0].close();
            connection.close();
        } catch (Exception ex) {
            log.info("数据库连接失败");
            ex.printStackTrace();
        }
        log.info("创建下月护理计划定时任务end:{}", new Date());
    }


    /**
     * 根据流程id创建下月护理计划插入数据库
     */
    @Override
    public void insertDatabaseNextMonth(String formInstId) {
        log.info("创建下月护理计划定时任务生成到数据库:{}", new Date());
        List<Map> dataList = new ArrayList<>();

        //获取日期是几号
        int dayOfMonth = firstDayOfNextMonth.getDayOfMonth();
        //获取月天数
        int daysInMonth = firstDayOfNextMonth.lengthOfMonth();
        try {
            Class.forName(driver);
            Connection connection = DriverManager.getConnection(url, username, password);
            final PreparedStatement[] ps = {null};
            String insertSql = "INSERT INTO `yida_plan_details`( `caregiver_form`, `caregiver_name`, `caregiver_member`, `caregiver_department`, " +
                    "`caregiver_card`, `caregiver_phone`, `department`, `care_object_form`, `care_object_name`, `care_object_card`, `care_object_level`, " +
                    "`care_object_address`, `care_object_longitude`, `care_object_latitude`, `within_the_week` , `care_start_time`, `care_end_time`, " +
                    "`execute_status`, `task_status`, `plan_id`, `care_object_id`, `certificate_type`, `is_choose`, `care_date_text` , `start_time_text` ," +
                    " `end_time_text` ,`form_inst_id`, `data_month`, `is_cogradient`, `start_time`, `end_time`, `is_delete`) " +
                    "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) ";
            ps[0] = connection.prepareStatement(insertSql);
            //创建护理计划  1次获取50条
            //根据实例ID  查询护理计划
            YDParam ydParam = YDParam.builder()
                    .formInstId(formInstId)
                    .build();
            Map formMap = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id).getFormData();
            List<Map> mapList = (List<Map>) formMap.get("tableField_lo3pmohx");
            for (int i = 0; i <= (daysInMonth - dayOfMonth); i++) {
                //获取是周几
                String dayOfWeek = firstDayOfNextMonth.plusDays(i).getDayOfWeek().getDisplayName(TextStyle.FULL, new Locale("zh", "CN"));
                //获取循环每天的日期
                LocalDate nextDay = firstDayOfNextMonth.plusDays(i);
                List<Map> maps = mapList.stream().filter(items -> items.get("selectField_lo3pmohy").equals(dayOfWeek.replace("星期", "周"))).collect(Collectors.toList());
                try {

                    for (Map map : maps) {
                        Object parse = JSON.parse(map.get("associationFormField_lo3pmohz_id").toString());
                        JSONArray jsonArray = JSON.parseArray(parse.toString());
                        JSONObject jsonObject = (JSONObject) jsonArray.get(0);
                        // 创建Statement对象，用于执行SQL语句
                        ps[0].setObject(1,map.get("associationFormField_lo3pmohz_id").toString());  //护理人员名称(关联表单)
                        ps[0].setObject(2, jsonObject.get("title"));  //护理人员名称
                        ps[0].setObject(3,map.get("employeeField_lo3pmoi6_id").toString());  //护理人员姓名-成员
                        ps[0].setObject(4,map.get("textField_lo3pmoi0"));  //护理人员所属机构
                        ps[0].setObject(5,map.get("textField_lomchzi3"));  //护理人员身份证
                        ps[0].setObject(6,map.get("textField_lo3pmoi3"));  //护理人员手机号
                        if (formMap.get("departmentSelectField_lxztlnnx_id") != null) {
                            ps[0].setObject(7,String.valueOf(formMap.get("departmentSelectField_lxztlnnx_id")));  //部门
                        }else {
                            ps[0].setObject(7,null);  //部门
                        }
                        ps[0].setObject(8,formMap.get("associationFormField_lo3pmohw_id").toString());  //护理对象姓名(关联表单)
                        ps[0].setObject(9,formMap.get("textField_lo55rupj"));  //护理对象姓名-文本
                        ps[0].setObject(10,formMap.get("textField_lnyhv5tn")); //护理对象身份证
                        ps[0].setObject(11,formMap.get("selectField_lnyhv5tp"));  //护理对象等级
                        ps[0].setObject(12,formMap.get("textField_lnyhv5tr"));  //护理对象住址
                        ps[0].setObject(13, formMap.get("textField_lvae99fj")); //护理对象经度
                        ps[0].setObject(14, formMap.get("textField_lvae99fk"));  //护理对象纬度
                        ps[0].setObject(15, dayOfWeek.replace("星期", "周"));  //护理时间  （星期几）
                        Date startDate = dateFormat.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3") + ":00");
                        Date endDate = dateFormat.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4") + ":00");
                        ps[0].setObject(16, startDate.getTime()); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                        ps[0].setObject(17, endDate.getTime()); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                        ps[0].setObject(18, "未执行");  //执行情况
                        ps[0].setObject(19, "待上报");  //任务状态
                        ps[0].setObject(20, map.get("textField_lo701pwy"));  //计划明细id
                        ps[0].setObject(21, formMap.get("textField_lop4v4qx"));  //护理对象ID
                        ps[0].setObject(22, map.get("textField_lo3pmoi4"));  //护理证书类型
                        ps[0].setObject(23, "是");  //是否可选
                        ps[0].setObject(24, String.valueOf(nextDay));  //护理日期-文本
                        ps[0].setObject(25, String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3")); //护理开始时间。 用当前日期+护理计划的  时分
                        ps[0].setObject(26, String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4")); //护理结束时间。 用当前日期+护理计划的  时分
                        ps[0].setObject(27, formInstId); //护理结束时间。 用当前日期+护理计划的  时分
                        ps[0].setObject(28, nextDataMonth); //下月数据日期
                        ps[0].setObject(29, "否"); //是否已同步宜搭
                        ps[0].setObject(30,map.get("selectField_lr1uxpu3"));
                        ps[0].setObject(31,map.get("selectField_lr1uxpu4"));
                        // 执行SQL查询
                        ps[0].setObject(32,0);
                        ps[0].addBatch();
                    }
                } catch (SQLException e) {
                    log.info("sql或插入数据库执行失败:{}",formMap.get("formInstanceId"));
                    e.printStackTrace();
                }
            }
            ps[0].executeBatch();
            ps[0].clearBatch();
            // 关闭资源
            ps[0].close();
            connection.close();
        } catch (Exception ex) {
            log.info("数据库连接失败");
            ex.printStackTrace();
        }
        log.info("创建下月护理计划定时任务end:{}", new Date());
    }
    @Override
    public void insertDatabaseTaskNew() {
        log.info("创建下月护理计划定时任务生成到数据库:{}", new Date());
        YDParam ydParam = new YDParam();
        String formId = getFormId(nextDataMonth);
        ydParam = YDParam.builder()
                .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")  //获取护理计划
//                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_m4hsya94", "未回写")))//护理对象暂停和正常、月度计划创建只查询正常的护理计划
                .build();
        List<Map> dataList = ydService.queryFormData_all(ydParam);
        try {
            List<Plan> planCopies = new ArrayList<>();
            Class.forName(driver);
            Connection connection = DriverManager.getConnection(url, username, password);
            final PreparedStatement[] ps = {null};
            String insertSql = "INSERT INTO `yida_plan_details_c`( `caregiver_form`, `caregiver_name`, `caregiver_member`, `caregiver_department`, " +
                    "`caregiver_card`, `caregiver_phone`, `department`, `care_object_form`, `care_object_name`, `care_object_card`, `care_object_level`, " +
                    "`care_object_address`, `care_object_longitude`, `care_object_latitude`, `within_the_week` , `care_start_time`, `care_end_time`, " +
                    "`execute_status`, `task_status`, `plan_id`, `care_object_id`, `certificate_type`, `is_choose`, `care_date_text` , `start_time_text` ," +
                    " `end_time_text` ,`form_inst_id`, `data_month`, `is_cogradient`, `start_time`, `end_time`, `is_delete`) " +
                    "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) ";
            dataList.forEach(dataItem -> {
//                Map formMap = (Map) dataItem.get("formData");
                List<Map> mapList = (List<Map>) dataItem.get("tableField_lo3pmohx");
                try {
                    for (Map map : mapList) {
                        ps[0] = connection.prepareStatement(insertSql);
                        Object parse = JSON.parse(map.get("associationFormField_lo3pmohz_id").toString());
                        JSONArray jsonArray = JSON.parseArray(parse.toString());
                        JSONObject jsonObject = (JSONObject) jsonArray.get(0);
                        // 创建Statement对象，用于执行SQL语句
                        ps[0].setObject(1,map.get("associationFormField_lo3pmohz_id").toString());  //护理人员名称(关联表单)
                        ps[0].setObject(2, jsonObject.get("title"));  //护理人员名称
                        ps[0].setObject(3,map.get("employeeField_lo3pmoi6_id").toString());  //护理人员姓名-成员
                        ps[0].setObject(4,map.get("textField_lo3pmoi0"));  //护理人员所属机构
                        ps[0].setObject(5,map.get("textField_lombve2g"));  //护理人员身份证
                        ps[0].setObject(6,map.get("textField_lo3pmoi3"));  //护理人员手机号
                        if (dataItem.get("departmentSelectField_lxzto1e1_id") != null) {
                            ps[0].setObject(7,String.valueOf(dataItem.get("departmentSelectField_lxzto1e1_id")));  //部门
                        }
                        ps[0].setObject(8,dataItem.get("associationFormField_lo3pmohw_id").toString());//护理对象姓名(关联表单)
                        ps[0].setObject(9,dataItem.get("textField_lo55rupj"));  //护理对象姓名-文本
                        ps[0].setObject(10,dataItem.get("textField_lnyhv5tn")); //护理对象身份证
                        ps[0].setObject(11,dataItem.get("selectField_lnyhv5tp"));  //护理对象等级
                        ps[0].setObject(12,dataItem.get("textField_lnyhv5tr"));  //护理对象住址
                        ps[0].setObject(13, dataItem.get("textField_lvae7q5n")); //护理对象经度
                        ps[0].setObject(14, dataItem.get("textField_lvae7q5o"));  //护理对象纬度
                        ps[0].setObject(15, dataItem.get("selectField_lo3pmohy"));  //护理时间  （星期几）
                        // 将字符串时间转换为Date对象
                        ps[0].setObject(16, dataItem.get("dateField_lred6eoj")); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                        ps[0].setObject(17, dataItem.get("dateField_lred6eok")); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                        ps[0].setObject(18, "未执行");  //执行情况
                        ps[0].setObject(19, "待上报");  //任务状态
                        ps[0].setObject(20, map.get("textField_lpgi8scj"));  //计划明细id
                        ps[0].setObject(21, dataItem.get("textField_lq9llmhl"));  //护理对象ID
                        ps[0].setObject(22, map.get("textField_lo3pmoi4"));  //护理证书类型
                        ps[0].setObject(23, "是");  //是否可选
                        ps[0].setObject(24, dataItem.get("textField_lrx0r38x"));  //护理日期-文本
                        ps[0].setObject(25, dateFormat.parse(dataItem.get("dateField_lred6eoj").toString())); //护理开始时间。 用当前日期+护理计划的  时分
                        ps[0].setObject(26, dateFormat.parse(dataItem.get("dateField_lred6eok").toString())); //护理结束时间。 用当前日期+护理计划的  时分
                        ps[0].setObject(27, dataItem.get("formInstanceId").toString()); //护理结束时间。 用当前日期+护理计划的  时分
                        ps[0].setObject(28, nextDataMonth); //下月数据日期
                        ps[0].setObject(29, "否"); //是否已同步宜搭
                        ps[0].setObject(30,map.get("selectField_lr1uxpu3"));//开始时间
                        ps[0].setObject(31,map.get("selectField_lr1uxpu4"));//结束时间
                        ps[0].setObject(32,0);
                        // 执行SQL查询
                        ps[0].addBatch();
//                        ps[0].execute();

//                        ydClient.operateData(YDParam.builder()
//                                .formInstanceId(dataItem.get("formInstanceId").toString())
//                                .updateFormDataJson(JSON.toJSONString(UtilMap.map("selectField_m4hsya94","已回写")))
//                                .useLatestVersion(true)
//                                .build(), YDConf.FORM_OPERATION.update);
                    }
                    ps[0].executeBatch();
                    ps[0].clearBatch();
                } catch (SQLException e) {
                    log.info("sql或插入数据库执行失败:{}",dataItem.get("formInstanceId"));
                    e.printStackTrace();
                } catch (ParseException e) {
                    throw new RuntimeException(e);
                }
            });
            // 关闭资源
            ps[0].close();
            connection.close();
        } catch (Exception ex) {
            log.info("数据库连接失败");
            ex.printStackTrace();
        }
        log.info("创建下月护理计划定时任务end:{}", new Date());
    }

    @Override
    public void insertHistory() {
        log.info("创建下月护理计划定时任务生成到数据库:{}", new Date());
        YDParam ydParam = new YDParam();
//        String formId = getFormId(nextDataMonth);
        ydParam = YDParam.builder()
                .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")  //获取护理计划
//                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_m4hsya94", "未回写")))//护理对象暂停和正常、月度计划创建只查询正常的护理计划
                .build();
        ydParam.setPageSize(1);
        long totalCount = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
        float pageSize = 50;
        ydParam.setCurrentPage(1);
        ydParam.setPageSize((int) pageSize);
        List<Map> dataList = new ArrayList<>();

//        dataList = ydService.queryFormData_all(ydParam);
        List<Plan> planCopies = new ArrayList<>();
        for (int page = 1; page <= Math.ceil(totalCount / pageSize); page++) {
            ydParam.setCurrentPage(page);
            //创建护理计划  1次获取50条
            dataList = (List<Map>) ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getData();

            dataList.forEach(e -> {
                Map formMap = (Map) e.get("formData");
                System.out.println(e);
                //护理人员
                Object parse = JSON.parse(formMap.get("associationFormField_lo3pmohz_id").toString());
                JSONArray jsonArray = JSON.parseArray(parse.toString());
                JSONObject jsonObject = (JSONObject) jsonArray.get(0);
                Plan planCopy = new Plan();
                planCopy.setDataMonth(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 7));
                //planCopy.setFormInstId();
                planCopy.setPlanInstId(e.get("formInstanceId").toString());
                planCopy.setCaregiverForm(formMap.get("associationFormField_lo3pmohz_id").toString());
                planCopy.setCaregiverName(jsonObject.get("title").toString());
                planCopy.setCaregiverMember(ObjectUtil.isNotNull(formMap.get("employeeField_lo3pmoi6_id"))?formMap.get("employeeField_lo3pmoi6_id").toString():"");
                planCopy.setCaregiverDepartment(formMap.get("textField_lo3pmoi0").toString());
                planCopy.setCaregiverCard(formMap.get("textField_lombve2g").toString());
                planCopy.setCaregiverPhone(formMap.get("textField_lo3pmoi3").toString());
                if (formMap.get("departmentSelectField_lxzto1e1_id") != null) {
                    planCopy.setDepartment(formMap.get("departmentSelectField_lxzto1e1_id").toString());
                }
                //护理对象
                parse = JSON.parse(formMap.get("associationFormField_lo3pmohw_id").toString());
                jsonArray = JSON.parseArray(parse.toString());
                jsonObject = (JSONObject) jsonArray.get(0);
                planCopy.setCareObjectForm(formMap.get("associationFormField_lo3pmohw_id").toString());
                planCopy.setCareObjectName(jsonObject.get("title").toString());
                planCopy.setCareObjectCard(formMap.get("textField_lnyhv5tn").toString());
                planCopy.setCareObjectLevel(ObjectUtil.isNotNull(formMap.get("selectField_lnyhv5tp"))?formMap.get("selectField_lnyhv5tp").toString():"");
                planCopy.setCareObjectAddress(ObjectUtil.isNotNull(formMap.get("textField_lnyhv5tr"))?formMap.get("textField_lnyhv5tr").toString():"");
                planCopy.setCareObjectLongitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5n"))?formMap.get("textField_lvae7q5n").toString():"");
                planCopy.setCareObjectLatitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5o"))?formMap.get("textField_lvae7q5o").toString():"");
                planCopy.setWithinTheWeek(formMap.get("selectField_lo3pmohy").toString());
                planCopy.setCareStartTime(Long.parseLong(formMap.get("dateField_lred6eoj").toString()));
                planCopy.setCareEndTime(Long.parseLong(formMap.get("dateField_lred6eok").toString()));
                planCopy.setStartTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(11, 16));
                planCopy.setEndTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))).substring(11, 16));
                if (ObjectUtil.isNotNull(formMap.get("dateField_lp7obkef"))) {
                    if (ObjectUtil.isNotNull(formMap.get("imageField_lrq19cti"))) {
                        parse = JSON.parse(formMap.get("imageField_lrq19cti").toString());
                        jsonArray = JSON.parseArray(parse.toString());
                        jsonObject = (JSONObject) jsonArray.get(0);
//                        "https://xvuk8m.aliwork.com" +
                        planCopy.setSignInPhoto(jsonObject.get("previewUrl").toString());
                        planCopy.setSignInPhotoDown(jsonObject.get("downloadUrl").toString());
                        planCopy.setSignInPhotoName(jsonObject.get("name").toString());
                    }
                    planCopy.setSignInLongitude(formMap.get("textField_ljghmc0e").toString());
                    planCopy.setSignInLatitude(formMap.get("textField_ljghmc0f").toString());
                    planCopy.setSignInTime(Long.parseLong(formMap.get("dateField_lp7obkef").toString()));
                    planCopy.setSignInTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lp7obkef").toString()))));
                    planCopy.setSignInPosition(formMap.get("textField_lrr62xq6").toString());
                    planCopy.setSignInDistance(ObjectUtil.isNotNull(formMap.get("numberField_lv4mhawr"))?formMap.get("numberField_lv4mhawr").toString():"");
                    planCopy.setClockInDate(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lp7obkef").toString()))).substring(0, 10));
                }
                if (ObjectUtil.isNotNull(formMap.get("dateField_lr36z6ha"))) {
                    if (ObjectUtil.isNotNull(formMap.get("imageField_lt893c3q"))){
                        parse = JSON.parse(formMap.get("imageField_lt893c3q").toString());
                        jsonArray = JSON.parseArray(parse.toString());
                        jsonObject = (JSONObject) jsonArray.get(0);
//                        "https://xvuk8m.aliwork.com" +
                        planCopy.setSignOutPhoto(jsonObject.get("previewUrl").toString());
                        planCopy.setSignOutPhotoDown(jsonObject.get("downloadUrl").toString());
                        planCopy.setSignOutPhotoName(jsonObject.get("name").toString());
                    }
                    planCopy.setSignOutLongitude(formMap.get("textField_lr36z6h6").toString());
                    planCopy.setSignOutLatitude(formMap.get("textField_lr36z6h8").toString());
                    planCopy.setSignOutTime(Long.parseLong(formMap.get("dateField_lr36z6ha").toString()));
                    planCopy.setSignOutTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lr36z6ha").toString()))));
                    planCopy.setSignOutPosition(formMap.get("textField_lrreodfe").toString());
                    planCopy.setSignOutDistance(ObjectUtil.isNotNull(formMap.get("numberField_lv4mhawi"))?formMap.get("numberField_lv4mhawi").toString():"");
                    planCopy.setClockOutDate(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lr36z6ha").toString()))).substring(0, 10));
                }
                planCopy.setExecuteStatus(formMap.get("selectField_lqcgsbw2").toString());
                planCopy.setTaskStatus(formMap.get("selectField_lo56u5fn").toString());
                planCopy.setPlanId(formMap.get("textField_lpgi8scj").toString());
                planCopy.setCareObjectId(formMap.get("textField_lq9llmhl").toString());
                planCopy.setCertificateType(ObjectUtil.isNotNull(formMap.get("textField_lo3pmoi4"))?formMap.get("textField_lo3pmoi4").toString():"");
                planCopy.setIsChoose(formMap.get("selectField_lr36z6hd").toString());
                planCopy.setIsDelete("0");
                planCopy.setIsCogradient("是");
                planCopy.setCareDateText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 10));
                planCopy.setStartTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))));
                planCopy.setEndTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))));

                planCopies.add(planCopy);

                try {
                    Thread.sleep(200);
                } catch (InterruptedException ex) {
                    throw new RuntimeException(ex);
                }
            });
            if (planCopies.size() > 0) {
                planService.saveBatch(planCopies);
                planCopies.clear();
            }
        }
    }
    @Override
    public void insertHistory2() {
//        Date date = new Date();
        long lastTime = 1734969600000L;
        List<String> dateList = new ArrayList<>();
        long startTime = 1732982400000L;
        dateList.add(String.valueOf(startTime));
        // 创建一个Calendar实例并设置为给定的时间戳
        Calendar calendar = Calendar.getInstance();
        calendar.setTimeInMillis(startTime);
        // 将Calendar的时间设置为第二天的0点
        calendar.add(Calendar.DAY_OF_MONTH, 1); // 增加一天
        calendar.set(Calendar.HOUR_OF_DAY, 0);  // 设置小时为0点
        calendar.set(Calendar.MINUTE, 0);       // 设置分钟为0
        calendar.set(Calendar.SECOND, 0);       // 设置秒为0
        calendar.set(Calendar.MILLISECOND, 0);  // 设置毫秒为0
        // 获取第二天0点的时间戳
        long secondDayTime = calendar.getTimeInMillis();
        dateList.add(String.valueOf(secondDayTime));
        log.info("创建下月护理计划定时任务生成到数据库:{}", new Date());
        while (startTime <= lastTime){
            YDParam ydParam = new YDParam();
            ydParam = YDParam.builder()
                    .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")  //获取护理计划
                    .searchFieldJson(JSON.toJSONString(UtilMap.map("dateField_lred6eoj, textField_lo3pmoi0", dateList, "上海市奉贤区海湾镇居家")))//护理对象暂停和正常、月度计划创建只查询正常的护理计划
                    .build();
            List<Map> dataList = ydService.queryFormData_all(ydParam);
            List<Plan> planCopies = new ArrayList<>();
            System.out.println(startTime+" - "+secondDayTime+"-"+dataList.size());
            if (ObjectUtil.isNotNull(dataList)&&dataList.size()>0) {
                dataList.forEach(formMap -> {
//                    Map formMap = (Map) e.get("formData");
//                    System.out.println(e);
                    //护理人员
                    Object parse = JSON.parse(formMap.get("associationFormField_lo3pmohz_id").toString());
                    JSONArray jsonArray = JSON.parseArray(parse.toString());
                    JSONObject jsonObject = (JSONObject) jsonArray.get(0);
                    Plan planCopy = new Plan();
                    planCopy.setDataMonth(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 7));
                    //planCopy.setFormInstId();
                    planCopy.setPlanInstId(formMap.get("formInstanceId").toString());
                    planCopy.setCaregiverForm(formMap.get("associationFormField_lo3pmohz_id").toString());
                    planCopy.setCaregiverName(jsonObject.get("title").toString());
                    planCopy.setCaregiverMember(ObjectUtil.isNotNull(formMap.get("employeeField_lo3pmoi6_id")) ? formMap.get("employeeField_lo3pmoi6_id").toString() : "");
                    planCopy.setCaregiverDepartment(formMap.get("textField_lo3pmoi0").toString());
                    planCopy.setCaregiverCard(formMap.get("textField_lombve2g").toString());
                    planCopy.setCaregiverPhone(formMap.get("textField_lo3pmoi3").toString());
                    if (formMap.get("departmentSelectField_lxzto1e1_id") != null) {
                        planCopy.setDepartment(formMap.get("departmentSelectField_lxzto1e1_id").toString());
                    }
                    //护理对象
                    parse = JSON.parse(formMap.get("associationFormField_lo3pmohw_id").toString());
                    jsonArray = JSON.parseArray(parse.toString());
                    jsonObject = (JSONObject) jsonArray.get(0);
                    planCopy.setCareObjectForm(formMap.get("associationFormField_lo3pmohw_id").toString());
                    planCopy.setCareObjectName(jsonObject.get("title").toString());
                    planCopy.setCareObjectCard(formMap.get("textField_lnyhv5tn").toString());
                    planCopy.setCareObjectLevel(ObjectUtil.isNotNull(formMap.get("selectField_lnyhv5tp")) ? formMap.get("selectField_lnyhv5tp").toString() : "");
                    planCopy.setCareObjectAddress(ObjectUtil.isNotNull(formMap.get("textField_lnyhv5tr")) ? formMap.get("textField_lnyhv5tr").toString() : "");
                    planCopy.setCareObjectLongitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5n")) ? formMap.get("textField_lvae7q5n").toString() : "");
                    planCopy.setCareObjectLatitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5o")) ? formMap.get("textField_lvae7q5o").toString() : "");
                    planCopy.setWithinTheWeek(formMap.get("selectField_lo3pmohy").toString());
                    planCopy.setCareStartTime(Long.parseLong(formMap.get("dateField_lred6eoj").toString()));
                    planCopy.setCareEndTime(Long.parseLong(formMap.get("dateField_lred6eok").toString()));
                    planCopy.setStartTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(11, 16));
                    planCopy.setEndTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))).substring(11, 16));
                    if (ObjectUtil.isNotNull(formMap.get("dateField_lp7obkef"))) {
                        if (ObjectUtil.isNotNull(formMap.get("imageField_lrq19cti"))) {
                            parse = JSON.parse(formMap.get("imageField_lrq19cti").toString());
                            jsonArray = JSON.parseArray(parse.toString());
                            jsonObject = (JSONObject) jsonArray.get(0);
//                        "https://xvuk8m.aliwork.com" +
                            planCopy.setSignInPhoto(jsonObject.get("previewUrl").toString());
                            planCopy.setSignInPhotoDown(jsonObject.get("downloadUrl").toString());
                            planCopy.setSignInPhotoName(jsonObject.get("name").toString());
                        }
                        planCopy.setSignInLongitude(formMap.get("textField_ljghmc0e").toString());
                        planCopy.setSignInLatitude(formMap.get("textField_ljghmc0f").toString());
                        planCopy.setSignInTime(Long.parseLong(formMap.get("dateField_lp7obkef").toString()));
                        planCopy.setSignInTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lp7obkef").toString()))));
                        planCopy.setSignInPosition(formMap.get("textField_lrr62xq6").toString());
                        planCopy.setSignInDistance(ObjectUtil.isNotNull(formMap.get("numberField_lv4mhawr")) ? formMap.get("numberField_lv4mhawr").toString() : "");
                        planCopy.setClockInDate(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lp7obkef").toString()))).substring(0, 10));
                    }
                    if (ObjectUtil.isNotNull(formMap.get("dateField_lr36z6ha"))) {
                        if (ObjectUtil.isNotNull(formMap.get("imageField_lt893c3q"))) {
                            parse = JSON.parse(formMap.get("imageField_lt893c3q").toString());
                            jsonArray = JSON.parseArray(parse.toString());
                            jsonObject = (JSONObject) jsonArray.get(0);
//                        "https://xvuk8m.aliwork.com" +
                            planCopy.setSignOutPhoto(jsonObject.get("previewUrl").toString());
                            planCopy.setSignOutPhotoDown(jsonObject.get("downloadUrl").toString());
                            planCopy.setSignOutPhotoName(jsonObject.get("name").toString());
                        }
                        planCopy.setSignOutLongitude(formMap.get("textField_lr36z6h6").toString());
                        planCopy.setSignOutLatitude(formMap.get("textField_lr36z6h8").toString());
                        planCopy.setSignOutTime(Long.parseLong(formMap.get("dateField_lr36z6ha").toString()));
                        planCopy.setSignOutTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lr36z6ha").toString()))));
                        planCopy.setSignOutPosition(formMap.get("textField_lrreodfe").toString());
                        planCopy.setSignOutDistance(ObjectUtil.isNotNull(formMap.get("numberField_lv4mhawi")) ? formMap.get("numberField_lv4mhawi").toString() : "");
                        planCopy.setClockOutDate(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lr36z6ha").toString()))).substring(0, 10));
                    }
                    planCopy.setExecuteStatus(formMap.get("selectField_lqcgsbw2").toString());
                    planCopy.setTaskStatus(formMap.get("selectField_lo56u5fn").toString());
                    planCopy.setPlanId(formMap.get("textField_lpgi8scj").toString());
                    planCopy.setCareObjectId(formMap.get("textField_lq9llmhl").toString());
                    planCopy.setCertificateType(ObjectUtil.isNotNull(formMap.get("textField_lo3pmoi4")) ? formMap.get("textField_lo3pmoi4").toString() : "");
                    planCopy.setIsChoose(formMap.get("selectField_lr36z6hd").toString());
                    planCopy.setIsDelete("0");
                    planCopy.setIsCogradient("是");
                    planCopy.setCareDateText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 10));
                    planCopy.setStartTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))));
                    planCopy.setEndTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))));

                    planCopies.add(planCopy);

                });
            }
            if (planCopies.size() > 0) {
                planService.saveBatch(planCopies);
                planCopies.clear();
            }
            startTime = secondDayTime;
            calendar.setTimeInMillis(secondDayTime);
            // 将Calendar的时间设置为第二天的0点
            calendar.add(Calendar.DAY_OF_MONTH, 1); // 增加一天
            calendar.set(Calendar.HOUR_OF_DAY, 0);  // 设置小时为0点
            calendar.set(Calendar.MINUTE, 0);       // 设置分钟为0
            calendar.set(Calendar.SECOND, 0);       // 设置秒为0
            calendar.set(Calendar.MILLISECOND, 0);  // 设置毫秒为0
            secondDayTime  = calendar.getTimeInMillis();
            dateList.clear();
            dateList.add(String.valueOf(startTime));
            dateList.add(String.valueOf(secondDayTime));
            try {
                Thread.sleep(100);
            } catch (InterruptedException ex) {
                throw new RuntimeException(ex);
            }

        }
    }

    @Override
    public void insertDatabaseOne(Map<String, Object> data) {

        YDParam ydParam = YDParam.builder()
                .formInstId(data.get("formInstId").toString())
                .build();
        Map formMap = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id).getFormData();
        List<Plan> planCopies = new ArrayList<>();
        //护理人员
        Object parse = JSON.parse(formMap.get("associationFormField_lo3pmohz_id").toString());
        JSONArray jsonArray = JSON.parseArray(parse.toString());
        JSONObject jsonObject = (JSONObject) jsonArray.get(0);
        Plan plan = new Plan();
        plan.setDataMonth(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 7));
        //planCopy.setFormInstId();
        plan.setPlanInstId(data.get("formInstId").toString());
        plan.setCaregiverForm(formMap.get("associationFormField_lo3pmohz_id").toString());
        plan.setCaregiverName(jsonObject.get("title").toString());
        plan.setCaregiverMember(ObjectUtil.isNotNull(formMap.get("employeeField_lo3pmoi6_id")) ? formMap.get("employeeField_lo3pmoi6_id").toString() : "");
        plan.setCaregiverDepartment(formMap.get("textField_lo3pmoi0").toString());
        plan.setCaregiverCard(formMap.get("textField_lombve2g").toString());
        plan.setCaregiverPhone(formMap.get("textField_lo3pmoi3").toString());
        if (formMap.get("departmentSelectField_lxzto1e1_id") != null) {
            plan.setDepartment(formMap.get("departmentSelectField_lxzto1e1_id").toString());
        }
        //护理对象
        parse = JSON.parse(formMap.get("associationFormField_lo3pmohw_id").toString());
        jsonArray = JSON.parseArray(parse.toString());
        jsonObject = (JSONObject) jsonArray.get(0);
        plan.setCareObjectForm(formMap.get("associationFormField_lo3pmohw_id").toString());
        plan.setCareObjectName(jsonObject.get("title").toString());
        plan.setCareObjectCard(formMap.get("textField_lnyhv5tn").toString());
        plan.setCareObjectLevel(ObjectUtil.isNotNull(formMap.get("selectField_lnyhv5tp")) ? formMap.get("selectField_lnyhv5tp").toString() : "");
        plan.setCareObjectAddress(ObjectUtil.isNotNull(formMap.get("textField_lnyhv5tr")) ? formMap.get("textField_lnyhv5tr").toString() : "");
        plan.setCareObjectLongitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5n")) ? formMap.get("textField_lvae7q5n").toString() : "");
        plan.setCareObjectLatitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5o")) ? formMap.get("textField_lvae7q5o").toString() : "");
        plan.setWithinTheWeek(formMap.get("selectField_lo3pmohy").toString());
        plan.setCareStartTime(Long.parseLong(formMap.get("dateField_lred6eoj").toString()));
        plan.setCareEndTime(Long.parseLong(formMap.get("dateField_lred6eok").toString()));
        plan.setStartTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(11, 16));
        plan.setEndTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))).substring(11, 16));

        plan.setExecuteStatus(formMap.get("selectField_lqcgsbw2").toString());
        plan.setTaskStatus(formMap.get("textField_m5adz7wx").toString());
        plan.setPlanId(formMap.get("textField_lpgi8scj").toString());
        plan.setCareObjectId(formMap.get("textField_lq9llmhl").toString());
        plan.setCertificateType(ObjectUtil.isNotNull(formMap.get("textField_lo3pmoi4")) ? formMap.get("textField_lo3pmoi4").toString() : "");
        plan.setIsChoose(formMap.get("selectField_lr36z6hd").toString());
        plan.setIsDelete("0");
        plan.setIsCogradient("是");
        plan.setCareDateText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 10));
        plan.setStartTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))));
        plan.setEndTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))));

        planCopies.add(plan);

//            });
//        }
        if (planCopies.size() > 0) {
            planService.saveBatch(planCopies);
            planCopies.clear();
        }

    }

    /**
     * 单条数据修改
     * @param data
     */
    @Override
    public void updateDatabaseOne(Map<String, Object> data) {
        Plan plan = new Plan();
        String id = data.get("id").toString();
        YDParam ydParam = YDParam.builder()
                .formInstId(data.get("formInstId").toString())
                .build();
        Map formMap = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id).getFormData();
        UpdateWrapper<Plan> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id",id);

        Map<String, Object> updateMap = new HashMap<>();
        Object parse = JSON.parse(formMap.get("associationFormField_lo3pmohz_id").toString());
        JSONArray jsonArray = JSON.parseArray(parse.toString());
        JSONObject jsonObject = (JSONObject) jsonArray.get(0);
        plan.setDataMonth(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 7));
        //planCopy.setFormInstId();
        plan.setPlanInstId(data.get("formInstId").toString());
        plan.setCaregiverForm(formMap.get("associationFormField_lo3pmohz_id").toString());
        plan.setCaregiverName(jsonObject.get("title").toString());
        plan.setCaregiverMember(ObjectUtil.isNotNull(formMap.get("textField_m58abv3j")) ? formMap.get("textField_m58abv3j").toString() : "");
        plan.setCaregiverDepartment(formMap.get("textField_lo3pmoi0").toString());
        plan.setCaregiverCard(formMap.get("textField_lombve2g").toString());
        plan.setCaregiverPhone(formMap.get("textField_lo3pmoi3").toString());
        //部门id
        if (formMap.get("textField_m58abv3l") != null) {
            plan.setDepartment(formMap.get("textField_m58abv3l").toString());
        }
        //护理对象
        parse = JSON.parse(formMap.get("associationFormField_lo3pmohw_id").toString());
        jsonArray = JSON.parseArray(parse.toString());
        jsonObject = (JSONObject) jsonArray.get(0);
        plan.setCareObjectForm(formMap.get("associationFormField_lo3pmohw_id").toString());
        plan.setCareObjectName(jsonObject.get("title").toString());
        plan.setCareObjectCard(formMap.get("textField_lnyhv5tn").toString());
        plan.setCareObjectLevel(ObjectUtil.isNotNull(formMap.get("selectField_lnyhv5tp")) ? formMap.get("selectField_lnyhv5tp").toString() : "");
        plan.setCareObjectAddress(ObjectUtil.isNotNull(formMap.get("textField_lnyhv5tr")) ? formMap.get("textField_lnyhv5tr").toString() : "");
        plan.setCareObjectLongitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5n")) ? formMap.get("textField_lvae7q5n").toString() : "");
        plan.setCareObjectLatitude(ObjectUtil.isNotNull(formMap.get("textField_lvae7q5o")) ? formMap.get("textField_lvae7q5o").toString() : "");
        plan.setWithinTheWeek(formMap.get("selectField_lo3pmohy").toString());
        plan.setCareStartTime(Long.parseLong(formMap.get("dateField_lred6eoj").toString()));
        plan.setCareEndTime(Long.parseLong(formMap.get("dateField_lred6eok").toString()));
        plan.setStartTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(11, 16));
        plan.setEndTime(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))).substring(11, 16));

        plan.setExecuteStatus(formMap.get("selectField_lqcgsbw2").toString());
        plan.setTaskStatus(formMap.get("textField_m5adz7wx").toString());
        plan.setPlanId(formMap.get("textField_lpgi8scj").toString());
        plan.setCareObjectId(formMap.get("textField_lq9llmhl").toString());
        plan.setCertificateType(ObjectUtil.isNotNull(formMap.get("textField_lo3pmoi4")) ? formMap.get("textField_lo3pmoi4").toString() : "");
        plan.setIsChoose(formMap.get("selectField_lr36z6hd").toString());
        plan.setIsDelete("0");
        plan.setIsCogradient("是");
        plan.setCareDateText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))).substring(0, 10));
        plan.setStartTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eoj").toString()))));
        plan.setEndTimeText(dateFormat.format(new Date(Long.parseLong(formMap.get("dateField_lred6eok").toString()))));
//        }
//        if (data.containsKey("planInstId")){
//            String planInstId = data.get("planInstId").toString();
//            ydClient.operateData(YDParam.builder()
//                    .formInstanceId(planInstId)
//                    .updateFormDataJson(JSON.toJSONString(updateMap))
//                    .useLatestVersion(true)
//                    .build(), YDConf.FORM_OPERATION.update);
//        }else {
//            plan.setIsCogradient("是");
//        }
        planMapper.update(plan,updateWrapper);
    }

    @Override
    public void deleteDatabaseOne(Map<String, Object> data) {
        Plan plan = new Plan();
        String id = data.get("id").toString();
        UpdateWrapper<Plan> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id",id);
        plan.setIsDelete("1");
        planMapper.update(plan,updateWrapper);

    }

    @Override
    public void afterDatabaseToYiDaTask() {
        log.info("开始执行同步宜搭是否同步和流程id到数据库");
        List<String> dataMonths = new ArrayList<>();
        dataMonths.add(dataMonth);
        dataMonths.add(nextDataMonth);
        Connection connection = null;
        PreparedStatement ps = null;
        for (int j = 0; j < dataMonths.size(); j++) {
            log.info("数据月份为：{}",dataMonths.get(j));
            String formId = getFormId(dataMonths.get(j));
            YDParam ydParam = YDParam.builder()
                    .formUuid(formId)
                    .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_m4hsya94", "未回写")))
                    .build();
            List<Map> dataList = ydService.queryFormData_all(ydParam);
            if(dataList.size()>0){
                log.info("已获取数据{}条，正在执行更新数据库",dataList.size());
                try {
                    Class.forName(driver);
                    connection = DriverManager.getConnection(url, username, password);
                    for (int h = 0; h < dataList.size(); h++) {
                        String updateSql = "UPDATE `yida_plan_details` SET is_cogradient = '是', plan_inst_id = '"+dataList.get(h).get("formInstanceId")+"' " +
                                "where data_month in ('" + dataMonth + "','"+nextDataMonth+"') AND is_cogradient = '否' AND id = '" + dataList.get(h).get("textField_m3wp95ge") + "';";
                        ps = connection.prepareStatement(updateSql);
                        ps.executeUpdate();
                        ydClient.operateData(YDParam.builder()
                                .formInstanceId(dataList.get(h).get("formInstanceId").toString())
                                .updateFormDataJson(JSON.toJSONString(UtilMap.map("selectField_m4hsya94", "已回写")))
                                .useLatestVersion(true)
                                .build(), YDConf.FORM_OPERATION.update);
                    }
                } catch (SQLException e) {
                    throw new RuntimeException(e);
                } catch (ClassNotFoundException e) {
                    throw new RuntimeException(e);
                }
            }}
    }

    @Override
    public void breakPersonUpdate(Map<String, Object> data) {
        long now = new Date().getTime();
        String card = data.get("card").toString();
        Plan plan = new Plan();
        UpdateWrapper<Plan> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("care_object_card",card);
        updateWrapper.ge("care_start_time",now);
        updateWrapper.eq("task_status","待上报");
        plan.setIsDelete("1");
        planMapper.update(plan,updateWrapper);

    }

    @Override
    public void testApi(String code) {
        LocalDate currentDate = LocalDate.now();
        int dayOfMonth = currentDate.getDayOfMonth();
        YDParam ydParam = new YDParam();

        ydParam = YDParam.builder()
                .formUuid("FORM-RK966E7105DFD27FA7EQHAOH9IFS2RN6MP3OL2")  //获取护理计划
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lo2b6bvf, textField_lryk5xa3", "正常",code)))//护理对象暂停和正常、月度计划创建只查询正常的护理计划
                .build();

        List<Map> dataList = ydService.queryFormData_all(ydParam);

        log.info("时间：{}",dayOfMonth);
//        if (dayOfMonth>24){
        log.info("开始");
        dataList.forEach(e->{
            deleteDatabaseInfo(e.get("formInstanceId").toString());
//                log.info("流程id:{}",e.get("formInstanceId").toString());
            log.info("护理对象:{}",e.get("textField_lo55rupj").toString());
            thisMonthDataBase(e.get("formInstanceId").toString());

//                insertDatabaseNextMonth(e.get("formInstanceId").toString());
        });
//        }
    }

    @Override
    public void execlExport(Map<String, Object> data, HttpServletResponse response, HttpServletRequest request) {
        if (data.isEmpty()){
            return;
        }
        QueryWrapper<Plan> planQuery = new QueryWrapper<>();
        data.forEach((k,v)->{
            if (k.equals("date_between")){
                Map<String,Object> formInstId = (Map<String,Object>) data.get("date_between");
                planQuery.ge("care_start_time",formInstId.get("start"));
                planQuery.le("care_start_time",formInstId.get("end"));
            }else{
                planQuery.eq(k,v);
            }
        });
        planQuery.eq("is_delete",0);
        planQuery.orderByAsc("care_start_time");
        List<Plan> records = planMapper.selectList(planQuery);
        if (ArrayUtil.isNotEmpty(records)){
            log.info("records：{}",records.toString());
        }

        //获取订单日期

        //获取文件标题
        String fileName = data.get("caregiver_department").toString() + data.get("care_date_text") + "护理计划";

        UtilExcel.exportMapAndListByTemplate(response, null, records, Plan.class, fileName, "Template.xlsx");
    }

    @Override
    public IPage<Plan> boardQuery(Map<String, Object> data) {
        QueryWrapper<Plan> planQuery = new QueryWrapper<>();
        IPage<Plan> planPage = new Page<>();
        data.forEach((k,v)->{
            if (k.equals("currentPage")){
                planPage.setCurrent(Long.parseLong(v.toString()));
            } else if (k.equals("pageSize")) {
                planPage.setSize(Long.parseLong(v.toString()));
            } else if (k.equals("date_between")){
                Map<String,Object> formInstId = (Map<String,Object>) data.get("date_between");
                planQuery.ge("care_start_time",formInstId.get("start"));
                planQuery.le("care_start_time",formInstId.get("end"));
            }else{
                planQuery.eq(k,v);
            }
        });
        planQuery.eq("is_delete",0);
        planQuery.orderByAsc("care_start_time");
        return planMapper.selectPage(planPage, planQuery);
    }

    @Override
    public IPage<Plan> mapQuery(Map<String, Object> data) {
        QueryWrapper<Plan> planQuery = new QueryWrapper<>();
        IPage<Plan> planPage = new Page<>();
        data.forEach((k,v)->{
            if (k.equals("currentPage")){
                planPage.setCurrent(Long.parseLong(v.toString()));
            } else if (k.equals("pageSize")) {
                planPage.setSize(Long.parseLong(v.toString()));
            } else if (k.equals("date_between")){
                Map<String,Object> formInstId = (Map<String,Object>) data.get("date_between");
                planQuery.ge("care_start_time",formInstId.get("start"));
                planQuery.le("care_start_time",formInstId.get("end"));
            } else if (k.equals("is_dep")) {

            } else{
                planQuery.eq(k,v);
            }
        });
        planQuery.eq("is_delete",0);
        if (data.containsKey("is_dep")){
            planQuery.isNotNull("clock_in_date");
            planQuery.isNull("clock_out_date");
            planQuery.orderByAsc("care_start_time");
        }
        return planMapper.selectPage(planPage, planQuery);
    }

    @Override
    public void testLocalDate() {

        // 创建一个 LocalDate 对象，表示 2025-01-09
        LocalDate localDate = LocalDate.of(2025, 1, 9);
        // 输出 LocalDate 对象的值
        log.info("LocalDate: " + localDate);

        // 获取并输出 day of month
        int dayOfMonth = localDate.getDayOfMonth();
        log.info("Day of Month: " + dayOfMonth);

    }

    @Override
    public void updateCareObjectInfo(String formInstId) {
        long now = new Date().getTime();
        Plan plan = new Plan();
        Map formData = ydClient.queryData(YDParam.builder()
                        .appType("APP_ERBDTFS82HOVBPL3NFH0")
                        .systemToken("RRB66F91T97H1WN89QZYC47PKLZO2ZQOUMOQLP")
                        .formInstanceId(formInstId)
                        .build(),
                YDConf.FORM_QUERY.retrieve_id).getFormData();
        UpdateWrapper<Plan> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("care_object_card",formData.get("textField_lnyhv5tn"));
        updateWrapper.ge("care_start_time",now);
        updateWrapper.eq("task_status","待上报");
        plan.setCareObjectLevel(formData.get("selectField_lnyhv5tp").toString());
        plan.setCareObjectAddress(formData.get("textField_lnyhv5tq").toString());
        plan.setCareObjectLongitude(formData.get("textField_ljghmc0e").toString());
        plan.setCareObjectLatitude(formData.get("textField_ljghmc0f").toString());
        planMapper.update(plan,updateWrapper);
    }

    /**
     * 定时任务同步数据库到宜搭
     */
    @Override
    public void databaseToYiDaTask() {
        List<String> dataMonths = new ArrayList<>();
        dataMonths.add(dataMonth);
        dataMonths.add(nextDataMonth);
        Connection connection = null;
        PreparedStatement ps = null;
        log.info("开始执行数据库更新到宜搭护理计划");
        ArrayList<Integer> ids = new ArrayList<>();
        for (int j = 0; j < dataMonths.size(); j++) {
            int times = 0;
            String formId = getFormId(dataMonths.get(j));
            try {
                Class.forName(driver);
                connection = DriverManager.getConnection(url, username, password);
                String selectSql = "SELECT `id`, `caregiver_form`,`caregiver_name`,`caregiver_member`,`caregiver_department`,`caregiver_card`," +
                        "`caregiver_phone`,`department`,`care_object_form`,`care_object_name`,`care_object_card`,`care_object_level`," +
                        "`care_object_address`,`care_object_longitude`,`care_object_latitude`,`within_the_week`,`care_start_time`," +
                        "`care_end_time`,`execute_status`,`task_status`,`plan_id`,`care_object_id`,`certificate_type`,`is_choose`," +
                        "`care_date_text`,`start_time_text`,`end_time_text`,`sign_in_photo`, `sign_in_photo_down`, `sign_in_photo_name`, `sign_out_photo`, " +
                        "`sign_out_photo_down`, `sign_out_photo_name`, `sign_in_longitude`, `sign_in_latitude`, `sign_in_time`, `sign_in_position`, " +
                        "`sign_out_longitude`, `sign_out_latitude`, `sign_out_time`, `sign_out_position`, `sign_in_distance`, `sign_out_distance` " +
                        "FROM `yida_plan_details` where  data_month = '" + dataMonths.get(j) + "' AND is_cogradient = '否' AND care_date_text > '2024-12-31' " +
                        "AND is_delete = '0' ORDER BY care_date_text ASC,id ASC LIMIT 400";
                ps = connection.prepareStatement(selectSql);
                ResultSet rs = ps.executeQuery(selectSql);
                while (rs.next()){
                    Map hashMap = new HashMap();
                    HashMap<String, Object> imageMap = new HashMap<>();
                    List<Map<String,Object>> images = new ArrayList<>();
                    hashMap.put("associationFormField_lo3pmohz", JSON.parse(rs.getString("caregiver_form")));  //护理人员名称
                    hashMap.put("textField_lo3pmoi0", rs.getString( "caregiver_department"));  //护理人员所属机构
                    hashMap.put("textField_lombve2g", rs.getString("caregiver_card"));  //护理人员身份证
                    hashMap.put("textField_lo3pmoi3", rs.getString("caregiver_phone"));  //护理人员手机号
                    hashMap.put("associationFormField_lo3pmohw", JSON.parse(rs.getString("care_object_form")));  //护理对象姓名
                    hashMap.put("textField_lnyhv5tn", rs.getString("care_object_card")); //护理对象身份证
                    hashMap.put("selectField_lnyhv5tp", rs.getString("care_object_level"));  //护理对象等级
                    hashMap.put("textField_lnyhv5tr", rs.getString("care_object_address"));  //护理对象住址
                    hashMap.put("selectField_lo3pmohy",rs.getString("within_the_week"));  //护理时间  （星期几）
                    hashMap.put("dateField_lred6eoj", rs.getString("care_start_time")); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                    hashMap.put("dateField_lred6eok", rs.getString("care_end_time")); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                    hashMap.put("selectField_lqcgsbw2", rs.getString("execute_status"));  //执行情况
                    hashMap.put("selectField_lo56u5fn", rs.getString("task_status"));  //任务状态

                    hashMap.put("textField_lvae7q5n", rs.getString("care_object_longitude")); //护理对象经度
                    hashMap.put("textField_lvae7q5o", rs.getString("care_object_latitude"));  //护理对象纬度

                    hashMap.put("textField_lrx0r38x", rs.getString("care_date_text"));  //护理日期-文本
                    hashMap.put("selectField_lr36z6hd", "是");  //任务状态
                    hashMap.put("textField_lo55rupj", rs.getString("care_object_name"));  //护理对象姓名-文本
                    hashMap.put("textField_lo3pmoi4", rs.getString("certificate_type"));  //护理证书类型
                    hashMap.put("employeeField_lo3pmoi6", JSON.parse(rs.getString("caregiver_member")));  //护理人员姓名-成员
                    hashMap.put("textField_lpgi8scj", rs.getString("plan_id"));  //计划明细id
                    hashMap.put("textField_lq9llmhl", rs.getString("care_object_id"));  //护理对象ID
                    if (rs.getString("department") != null) {
                        hashMap.put("departmentSelectField_lxzto1e1", rs.getString("department"));  //部门
                    }
                    hashMap.put("textField_m3wp95ge",rs.getInt("id"));//数据库id
                    imageMap.put("downloadUrl",rs.getString("sign_in_photo_down"));
                    imageMap.put("name",rs.getString("sign_in_photo_name"));
                    imageMap.put("previewUrl",rs.getString("sign_in_photo"));
                    imageMap.put("url",rs.getString("sign_in_photo_down"));
                    images.add(imageMap);
                    //, `sign_in_photo_down`, `sign_in_photo_name`, `sign_out_photo`, `sign_out_photo_down`, `sign_out_photo_name`
                    hashMap.put("textField_ljghmc0e",rs.getString("sign_in_longitude"));
                    hashMap.put("textField_ljghmc0f",rs.getString("sign_in_latitude"));
                    hashMap.put("dateField_lp7obkef",rs.getString("sign_in_time"));
                    hashMap.put("textField_lrr62xq6",rs.getString("sign_in_position"));
                    hashMap.put("selectField_lqcgsbw2",rs.getString("execute_status"));
                    hashMap.put("imageField_lrq19cti",JSON.toJSONString(images));
                    images.clear();

                    imageMap.put("downloadUrl",rs.getString("sign_out_photo_down"));
                    imageMap.put("name",rs.getString("sign_out_photo_name"));
                    imageMap.put("previewUrl",rs.getString("sign_out_photo"));
                    imageMap.put("url",rs.getString("sign_out_photo_down"));
                    images.add(imageMap);

                    hashMap.put("textField_lr36z6h6",rs.getString("sign_out_longitude"));
                    hashMap.put("textField_lr36z6h8",rs.getString("sign_out_latitude"));
                    hashMap.put("dateField_lr36z6ha",rs.getString("sign_out_time"));
                    hashMap.put("textField_lrreodfe",rs.getString("sign_out_position"));
                    hashMap.put("imageField_lt893c3q",JSON.toJSONString(images));
                    hashMap.put("numberField_lv4mhawr",rs.getString("sign_in_distance"));
                    hashMap.put("numberField_lv4mhawi",rs.getString("sign_out_distance"));
                    hashMap.put("selectField_m4hsya94", "未回写");  //回写数据库状态


                    times++;
                    ids.add(rs.getInt("id"));

                    if (times%200==0){
                        Thread.sleep(4000);
                    }
                    ydClient.operateData(YDParam.builder()
                            .formUuid(formId)
                            .searchCondition(JSON.toJSONString(UtilMap.map("textField_m3wp95ge",rs.getInt("id"))))
                            .formDataJson(JSON.toJSONString(hashMap))
                            .build(), YDConf.FORM_OPERATION.upsert);
                }
                //关闭资源
                ps.close();
                connection.close();
            } catch (SQLException e) {
                log.info("SQL执行错误");
            } catch (ClassNotFoundException e) {
                log.info("驱动注册失败");
            } catch (InterruptedException e) {
                throw new RuntimeException(e);
            }
        }
    }

    /**
     * 根据护理计划安排流程id逻辑删除数据库当月护理计划
     * @param formInstId 护理计划安排流程id
     */
    @Override
    public void deleteDatabaseInfo(String formInstId) {

        log.info("开始逻辑删除下月数据库护理计划");
        try {
            Class.forName(driver);
            Connection connection = DriverManager.getConnection(url, username, password);
            PreparedStatement ps = null;
//            String deleteSql = "DELETE FROM`yida_plan_details` where data_month in ('" + dataMonth + "','"+nextDataMonth+"') AND form_inst_id = '"+formInstId+"' AND task_status = '待上报'";
            String deleteSql = "UPDATE `yida_plan_details` SET is_delete = 1 where data_month in ('" + dataMonth + "','"+nextDataMonth+"') AND form_inst_id = '"+formInstId+"' AND task_status = '待上报'";
            ps = connection.prepareStatement(deleteSql);
            ps.executeUpdate(deleteSql);

            ps.close();
            connection.close();
        } catch (SQLException e) {
            log.info("SQL执行错误");
            throw new RuntimeException(e);
        } catch (ClassNotFoundException e) {
            log.info("驱动注册失败");
        }
    }

    @SneakyThrows
    @Override
    public void deleteBothInfo() {

        Connection connection = null;
        PreparedStatement ps = null;
        log.info("开始执行数据库更新到宜搭护理计划");
        try {
            Class.forName(driver);
            connection = DriverManager.getConnection(url, username, password);
            String selectSql = "SELECT `plan_inst_id` FROM`yida_plan_details` where is_delete = 1 ";
            ps = connection.prepareStatement(selectSql);
            ResultSet rs = ps.executeQuery(selectSql);
            String deleteSql = "DELETE FROM`yida_plan_details` where is_delete = 1 ";
            ps = connection.prepareStatement(deleteSql);
            ps.executeUpdate();
            while (rs.next()){
                log.info("删除流程id:{}",rs.getString("plan_inst_id"));
                //删除
                ydClient.operateData(YDParam.builder()
                        .formInstanceId(rs.getString("plan_inst_id"))
                        .build(), YDConf.FORM_OPERATION.delete);
            }
        } catch (ClassNotFoundException | SQLException e) {
            e.printStackTrace();
        }finally {
            connection.close();
            ps.close();
        }
    }

    /**
     * 生成当月数据库未上报护理计划
     * @param formInstId
     */
    @Override
    public void thisMonthDataBase(String formInstId){
        log.info("开始生成当月数据库未上报护理计划:{}",new Date());
        //根据实例ID  查询护理计划
        YDParam ydParam = YDParam.builder()
                .formInstId(formInstId)
                .build();
        Map formData = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id).getFormData();
        log.info("formData{}",formData);
        List<Map> mapList = (List<Map>) formData.get("tableField_lo3pmohx");
        Connection connection = null;
        //获取当前日期是本月几号
        LocalDateTime now = LocalDateTime.now();
        int dayOfMonth = now.getDayOfMonth();
//        log.info("currentDate:{}",currentDate);
        log.info("now:{}",now);
        log.info("dayOfMonth:{}",dayOfMonth);
        //获取本月天数
        int daysInMonth = now.toLocalDate().lengthOfMonth();
        log.info("daysInMonth:{}",daysInMonth);

        try {
            Class.forName(driver);
            connection = DriverManager.getConnection(url, username, password);
            final PreparedStatement[] ps = {null};
            String insertSql = "INSERT INTO `yida_plan_details`( `caregiver_form`, `caregiver_name`, `caregiver_member`, `caregiver_department`, " +
                    "`caregiver_card`, `caregiver_phone`, `department`, `care_object_form`, `care_object_name`, `care_object_card`, `care_object_level`, " +
                    "`care_object_address`, `care_object_longitude`, `care_object_latitude`, `within_the_week` , `care_start_time`, `care_end_time`, " +
                    "`execute_status`, `task_status`, `plan_id`, `care_object_id`, `certificate_type`, `is_choose`, `care_date_text` , `start_time_text` ," +
                    " `end_time_text` ,`form_inst_id`, `data_month`, `is_cogradient`, `start_time`, `end_time`, `is_delete`) " +
                    "VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?) ";
            ps[0] = connection.prepareStatement(insertSql);
            for (int i = 1; i <= daysInMonth - dayOfMonth; i++) {
                //获取当前日期是本月几号
                //获取是周几
                String dayOfWeek = now.plusDays(i).getDayOfWeek().getDisplayName(TextStyle.FULL, new Locale("zh", "CN"));
                //获取循环每天的日期
                LocalDate nextDay = LocalDate.from(now.plusDays(i));
                List<Map> maps = mapList.stream().filter(item -> item.get("selectField_lo3pmohy").equals(dayOfWeek.replace("星期", "周"))).collect(Collectors.toList());
                for (Map map : maps) {
                    // 创建Statement对象，用于执行SQL语句
                    JSONArray jsonArray = JSON.parseArray(JSON.parse(map.get("associationFormField_lo3pmohz_id").toString()).toString());
                    JSONObject jsonObject = (JSONObject) jsonArray.get(0);
                    log.info("护理人员名称(关联表单)：{}",map.get("associationFormField_lo3pmohz_id").toString());

                    ps[0].setObject(1, map.get("associationFormField_lo3pmohz_id").toString());  //护理人员名称(关联表单)
                    ps[0].setObject(2, jsonObject.get("title"));  //护理人员名称
                    ps[0].setObject(3, map.get("employeeField_lo3pmoi6_id").toString());  //护理人员姓名-成员
                    ps[0].setObject(4, map.get("textField_lo3pmoi0"));  //护理人员所属机构
                    ps[0].setObject(5, map.get("textField_lomchzi3"));  //护理人员身份证
                    ps[0].setObject(6, map.get("textField_lo3pmoi3"));  //护理人员手机号
                    if (formData.get("departmentSelectField_lxztlnnx_id") != null) {
                        ps[0].setObject(7, String.valueOf(formData.get("departmentSelectField_lxztlnnx_id")));  //部门
                    }else {
                        ps[0].setObject(7,null);
                    }
                    log.info("护理对象姓名(关联表单):{}",formData.get("associationFormField_lo3pmohw_id").toString());
                    ps[0].setObject(8, formData.get("associationFormField_lo3pmohw_id").toString());  //护理对象姓名(关联表单)
                    ps[0].setObject(9, formData.get("textField_lo55rupj"));  //护理对象姓名-文本
                    ps[0].setObject(10, formData.get("textField_lnyhv5tn")); //护理对象身份证
                    ps[0].setObject(11, formData.get("selectField_lnyhv5tp"));  //护理对象等级
                    ps[0].setObject(12, formData.get("textField_lnyhv5tr"));  //护理对象住址
                    ps[0].setObject(13, formData.get("textField_lvae99fj")); //护理对象经度
                    ps[0].setObject(14, formData.get("textField_lvae99fk"));  //护理对象纬度
                    ps[0].setObject(15, dayOfWeek.replace("星期", "周"));  //护理时间  （星期几）
                    Date startDate = dateFormat.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3") + ":00");
                    Date endDate = dateFormat.parse(String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4") + ":00");
                    ps[0].setObject(16, startDate.getTime()); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                    ps[0].setObject(17, endDate.getTime()); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                    ps[0].setObject(18, "未执行");  //执行情况
                    ps[0].setObject(19, "待上报");  //任务状态
                    ps[0].setObject(20, map.get("textField_lo701pwy"));  //计划明细id
                    ps[0].setObject(21, formData.get("textField_lop4v4qx"));  //护理对象ID
                    ps[0].setObject(22, map.get("textField_lo3pmoi4"));  //护理证书类型
                    ps[0].setObject(23, "是");  //是否可选
                    ps[0].setObject(24, String.valueOf(nextDay));  //护理日期-文本
                    ps[0].setObject(25, String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu3")); //护理开始时间。 用当前日期+护理计划的  时分
                    ps[0].setObject(26, String.valueOf(nextDay) + " " + map.get("selectField_lr1uxpu4")); //护理结束时间。 用当前日期+护理计划的  时分
                    ps[0].setObject(27, formInstId); //护理结束时间。 用当前日期+护理计划的  时分
                    ps[0].setObject(28, dataMonth); //当月数据日期
                    ps[0].setObject(29, "否"); //是否已同步宜搭
                    ps[0].setObject(30,map.get("selectField_lr1uxpu3"));
                    ps[0].setObject(31,map.get("selectField_lr1uxpu4"));
                    ps[0].setObject(32,0);
                    // 执行SQL查询
                    ps[0].addBatch();
                    log.info("sql:{}", ps[0].toString());
                }
            }
            ps[0].executeBatch();
            ps[0].close();
            connection.close();
        } catch (SQLException e) {
            throw new RuntimeException(e);
        } catch (ClassNotFoundException e) {
            throw new RuntimeException(e);
        } catch (ParseException e) {
            throw new RuntimeException(e);
        }

    }

    /**
     * 生成当月宜搭未上报护理计划
     * @param formInstId
     */
    public void thisMonthDataBaseToYiDa(String formInstId){
        log.info("开始生成当月和下月宜搭未上报护理计划");
        String formId = getFormId(dataMonth);
        Connection connection = null;
        PreparedStatement ps = null;
        log.info("开始执行数据库更新到宜搭护理计划");
        try {
            Class.forName(driver);
            connection = DriverManager.getConnection(url, username, password);
            String selectSql = "SELECT `id`, `caregiver_form`,`caregiver_name`,`caregiver_member`,`caregiver_department`,`caregiver_card`," +
                    "`caregiver_phone`,`department`,`care_object_form`,`care_object_name`,`care_object_card`,`care_object_level`," +
                    "`care_object_address`,`care_object_longitude`,`care_object_latitude`,`within_the_week`,`care_start_time`," +
                    "`care_end_time`,`execute_status`,`task_status`,`plan_id`,`care_object_id`,`certificate_type`,`is_choose`," +
                    "`care_date_text`,`start_time_text`,`end_time_text` FROM`yida_plan_details` " +
                    "WHERE data_month IN  ('" + dataMonth + "','" + nextDataMonth + "') AND is_cogradient = '否' AND form_inst_id = '"+formInstId+"'";
            ps = connection.prepareStatement(selectSql);
            ResultSet rs = ps.executeQuery(selectSql);
            List<Integer> ids = new ArrayList<Integer>();
            while (rs.next()){


                Map hashMap = new HashMap();

                hashMap.put("associationFormField_lo3pmohz", JSON.parse(rs.getString("caregiver_form")));  //护理人员名称
                hashMap.put("textField_lo3pmoi0", rs.getString( "caregiver_department"));  //护理人员所属机构
                hashMap.put("textField_lombve2g", rs.getString("caregiver_card"));  //护理人员身份证
                hashMap.put("textField_lo3pmoi3", rs.getString("caregiver_phone"));  //护理人员手机号
                hashMap.put("associationFormField_lo3pmohw", JSON.parse(rs.getString("care_object_form")));  //护理对象姓名
                hashMap.put("textField_lnyhv5tn", rs.getString("care_object_card")); //护理对象身份证
                hashMap.put("selectField_lnyhv5tp", rs.getString("care_object_level"));  //护理对象等级
                hashMap.put("textField_lnyhv5tr", rs.getString("care_object_address"));  //护理对象住址
                hashMap.put("selectField_lo3pmohy",rs.getString("within_the_week"));  //护理时间  （星期几）
                hashMap.put("dateField_lred6eoj", rs.getString("care_start_time")); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                hashMap.put("dateField_lred6eok", rs.getString("care_end_time")); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                hashMap.put("selectField_lqcgsbw2", "未执行");  //执行情况
                hashMap.put("selectField_lo56u5fn", "待上报");  //任务状态

                hashMap.put("textField_lvae7q5n", rs.getString("care_object_longitude")); //护理对象经度
                hashMap.put("textField_lvae7q5o", rs.getString("care_object_latitude"));  //护理对象纬度

                hashMap.put("textField_lrx0r38x", rs.getString("care_date_text"));  //护理日期-文本
                hashMap.put("selectField_lr36z6hd", "是");  //任务状态
                hashMap.put("textField_lo55rupj", rs.getString("care_object_name"));  //护理对象姓名-文本
                hashMap.put("textField_lo3pmoi4", rs.getString("certificate_type"));  //护理证书类型
                hashMap.put("employeeField_lo3pmoi6", JSON.parse(rs.getString("caregiver_member")));  //护理人员姓名-成员
                hashMap.put("textField_lpgi8scj", rs.getString("plan_id"));  //计划明细id
                hashMap.put("textField_lq9llmhl", rs.getString("care_object_id"));  //护理对象ID
                if (rs.getString("department") != null) {
                    hashMap.put("departmentSelectField_lxzto1e1", rs.getString("department"));  //部门
                }
                hashMap.put("textField_m3wp95ge",rs.getInt("id"));//数据库id
                ids.add(rs.getInt("id"));
                ydClient.operateData(YDParam.builder()
                        .formUuid(formId)
                        .formDataJson(JSON.toJSONString(hashMap))
                        .build(), YDConf.FORM_OPERATION.create);
            }
            if (ObjectUtil.isNotNull(ids)&&ids.size()>0){
                for (int i = 0; i < ids.size(); i++) {
                    YDParam ydParam = YDParam.builder()
                            .formUuid("FORM-RK966E7105DFD27FA7EQHAOH9IFS2RN6MP3OL2")
                            .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_m3wp95ge",ids.get(i))))
                            .build();
                    List<Map> dataList = (List<Map>) ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getData();
                    if(dataList.size()>0){
                        Map map = (Map) dataList.get(0);
                        String updateSql = "UPDATE `yida_plan_details` SET is_cogradient = '是', plan_inst_id = '"+map.get("formInstanceId")+"'" +
                                " where data_month IN  ('" + dataMonth + "','" + nextDataMonth + "') AND is_cogradient = '否' AND id = '" + ids.get(i) + "'";
                        ps = connection.prepareStatement(updateSql);
                        ps.executeUpdate();
                    }
                }
            }
            //关闭资源
            ps.close();
            connection.close();
        } catch (SQLException e) {
            log.info("SQL执行错误");
        } catch (ClassNotFoundException e) {
            log.info("驱动注册失败");
        }

    }


    @Override
    public void updateFromClock(Map<String, Object> data) {
        Plan plan = new Plan();
        String id = data.get("id").toString();
        UpdateWrapper<Plan> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("id",id);

        List<Map<String,Object>> image = (List<Map<String, Object>>) data.get("imageField_lp7obkee");
//        String remoteUrl = "https://xvuk8m.aliwork.com" + image.get(0).get("previewUrl");
        String remoteUrl = image.get(0).get("previewUrl").toString();
        String fileName = image.get(0).get("name").toString();
        String downLoad = image.get(0).get("downloadUrl").toString();
//            downloadFile(remoteUrl,localUrl+fileName);
        Date date = new Date(Long.parseLong(data.get("dateField_lp7obkef").toString()));
        Map<String, Object> updateMap = new HashMap<>();
        if ("签到".equals(data.get("textField_lqoldqro"))){
            plan.setSignInLongitude(data.get("textField_lu2ewbl7").toString());//签到经度
            plan.setSignInLatitude(data.get("textField_lu2ewbl8").toString());//签到维度
            plan.setSignInTime(Long.parseLong(data.get("dateField_lp7obkef").toString()));//签到时间
            plan.setSignInPosition(data.get("textField_lrr62xq6").toString());//签到位置信息
            plan.setExecuteStatus("执行中");
            plan.setClockInDate(dateFormat.format(date));//打卡时间文本
            plan.setSignInPhoto(remoteUrl);
            plan.setSignInPhotoName(fileName);
            plan.setSignInPhotoDown(downLoad);
            plan.setSignInDistance(data.get("numberField_luw22bp4").toString());
            updateMap.put("textField_ljghmc0e",data.get("textField_lu2ewbl7"));
            updateMap.put("textField_ljghmc0f",data.get("textField_lu2ewbl8"));
            updateMap.put("dateField_lp7obkef",data.get("dateField_lp7obkef"));
            updateMap.put("textField_lrr62xq6",data.get("textField_lrr62xq6"));
            updateMap.put("selectField_lqcgsbw2",data.get("执行中"));
            updateMap.put("imageField_lrq19cti",data.get("imageField_lp7obkee"));
            updateMap.put("numberField_lv4mhawr",data.get("numberField_luw22bp4"));
        } else if ("签退".equals(data.get("textField_lqoldqro"))) {
            plan.setSignOutLongitude(data.get("textField_lu2ewbl7").toString());//签到经度
            plan.setSignOutLatitude(data.get("textField_lu2ewbl8").toString());//签到维度
            plan.setSignOutTime(Long.parseLong(data.get("dateField_lp7obkef").toString()));//签到时间
            plan.setSignOutPosition(data.get("textField_lrr62xq6").toString());//签到位置信息
            plan.setExecuteStatus("已完成");
            plan.setClockOutDate(dateFormat.format(date));//打卡时间文本
            plan.setSignOutPhoto(remoteUrl);
            plan.setSignOutPhotoName(fileName);
            plan.setSignOutPhotoDown(downLoad);
            plan.setSignOutDistance(data.get("numberField_luw22bp4").toString());
            updateMap.put("textField_lr36z6h6",data.get("textField_lu2ewbl7"));
            updateMap.put("textField_lr36z6h8",data.get("textField_lu2ewbl8"));
            updateMap.put("dateField_lr36z6ha",data.get("dateField_lp7obkef"));
            updateMap.put("textField_lrreodfe",data.get("textField_lrr62xq6"));
            updateMap.put("selectField_lqcgsbw2",data.get("已完成"));
            updateMap.put("imageField_lt893c3q",data.get("imageField_lp7obkee"));
            updateMap.put("numberField_lv4mhawi",data.get("numberField_luw22bp4"));
        }
        if (data.containsKey("planInstId")){
            String planInstId = data.get("planInstId").toString();
            ydClient.operateData(YDParam.builder()
                    .formInstanceId(planInstId)
                    .updateFormDataJson(JSON.toJSONString(updateMap))
                    .useLatestVersion(true)
                    .build(), YDConf.FORM_OPERATION.update);
        }else {
            plan.setIsCogradient("否");
        }
        planMapper.update(plan,updateWrapper);
    }

    @Override
    public void updateDBStatus() {
        String date = DateUtil.date().toString("yyyy-MM-dd");
        UpdateWrapper<Plan> updateWrapper = new UpdateWrapper<>();
        updateWrapper.eq("care_date_text",date);
        Plan plan = new Plan();
        plan.setTaskStatus("已上报");
        planMapper.update(plan,updateWrapper);

    }

    @Override
    public void deleteClockIn() {
        long time = DateUtil.offsetDay(DateUtil.date(), -30).getTime();
        List<String> dateList = new ArrayList<>();
        dateList.add(null);
        dateList.add(String.valueOf(time));
        YDParam ydParam = YDParam.builder()
                .appType("APP_YIDJ71B8QORD2YHNZ26Q")
                .systemToken("RH766AC1707FCUZQAKET09YV8I8K3VZA1GVNLN2")
                .formUuid("FORM-1683937473624B108E28436FB8A06129K78N")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("dateField_lp7obkef", dateList)))
                .build();
        List<Map> allList = ydService.queryFormData_all(ydParam);

        allList.forEach(e->{
            ydClient.operateData(YDParam.builder()
                    .formInstanceId(e.get("formInstanceId").toString())
                    .build(), YDConf.FORM_OPERATION.delete);
        });
    }

    @Override
    public String getFormId(String month){
        log.info("获取{}表单id",month);
        YDParam ydParamDelete = YDParam.builder()
                .formUuid("FORM-E22142AB07F643C4A9E2EA3A5A48B3C9RUH8")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("textField_m3wgvuuu",month)))
                .build();

        List<Map> allFormData = ydService.queryFormData_all(ydParamDelete);
        if (ObjectUtil.isNotNull(allFormData) && allFormData.size() > 0){
            log.info("当月{}表单id为：{}",month,allFormData.get(0).get("textField_m3wgvuuv"));
            return allFormData.get(0).get("textField_m3wgvuuv").toString();
        }else {
            log.info("未获取到{}月份对应的表单id,请检查护理计划配置表",month);
            return null;
        }
    }

    /**
     * 每天同步护理状态
     */
    @Override
    public void syncUpdatePlanStatus() {
        log.info("每天同步护理状态开始{}", new Date());
        syncPlanStatus();
        log.info("每天同步护理状态结束{}", new Date());
    }


    /**
     * 不同上报状态  定时任务每天执行
     */
    @SneakyThrows
    public void syncPlanStatus() {
        String formId = getFormId(dataMonth);

        LocalDate currentDate = LocalDate.now();
        //获取需要更新的护理任务   护理日期是当天      且状态为未上报
        YDParam ydParam = YDParam.builder()
                .formUuid(formId)
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lo56u5fn, dateField_lred6eoj","待上报",Arrays.asList(UtilDateTime.parse( currentDate+ " 00:00:59","yyyy-MM-dd HH:mm:ss"),UtilDateTime.parse( currentDate+ " 23:59:59","yyyy-MM-dd HH:mm:ss")))))
                .build();
        //pagesize设为1获取总数
        ydParam.setPageSize(1);
        long totalCount = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getTotalCount();
        log.info("totalCount{}",totalCount);

        //如果条数不为0 就一直递归
        if (totalCount>0){
            ydParam.setPageSize(50);
            List<Map> dataList = new ArrayList<>();
            dataList = (List<Map>) ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_search_form).getData();
            log.info("dataList:{}",dataList.size());
            //循环处理修改状态为已上报
            dataList.forEach(item -> {
                //修改状态为已上报
                try {
                    ydClient.operateData(YDParam.builder()
                            .formInstanceId(item.get("formInstanceId").toString())
                            .updateFormDataJson(JSON.toJSONString(UtilMap.map("selectField_lo56u5fn","已上报")))
                            .useLatestVersion(true)
                            .build(), YDConf.FORM_OPERATION.update);
                } catch (Exception e) {
                    log.info("异常流程ID为：{}",item.get("formInstanceId").toString());
                    e.printStackTrace();
                }
            });
            Thread.sleep(5000);
            syncUpdatePlanStatus();
        }
    }

    /**
     * 新增护理计划1条  生成本月护理数据
     * @param formInstId
     */
    @Override
    public void creatInfo(String formInstId){
        log.info("新增护理计划:{}",new Date());
        //根据实例ID  查询护理计划
        log.info("护理计划安排ID:{}",formInstId);
        YDParam ydParam = YDParam.builder()
                .formInstId(formInstId)
                .build();
        Map formData = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id).getFormData();
        log.info("formData{}",formData);
        List<Map> mapList = (List<Map>) formData.get("tableField_lo3pmohx");

        LocalDate currentDate = LocalDate.now();
        //获取当前日期是本月几号
        int dayOfMonth = currentDate.getDayOfMonth();
        //获取本月天数
        int daysInMonth = currentDate.lengthOfMonth();

        for (int i = 0; i <=daysInMonth-dayOfMonth ; i++) {
            //获取当前日期是本月几号
            LocalDate currentDay = LocalDate.now();
            //获取是周几
            String dayOfWeek = currentDay.plusDays(i).getDayOfWeek().getDisplayName(TextStyle.FULL, new Locale("zh", "CN"));
            //获取循环每天的日期
            LocalDate nextDay = currentDate.plusDays(i);
            List<Map> maps =  mapList.stream().filter(item -> item.get("selectField_lo3pmohy").equals(dayOfWeek.replace("星期","周"))).collect(Collectors.toList());
            for (Map map : maps) {
                //通过护理时间  护理对象ID    证数据是否已经存在
                YDParam ydParam1 = YDParam.builder()
                        .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")
                        .searchFieldJson(JSON.toJSONString(UtilMap.map("dateField_lred6eoj, textField_lnyhv5tn, textField_lombve2g",Arrays.asList(UtilDateTime.parse( nextDay+ " 00:00:01","yyyy-MM-dd HH:mm:ss"),UtilDateTime.parse( nextDay+ " 23:59:59","yyyy-MM-dd HH:mm:ss")),formData.get("textField_lnyhv5tn"),map.get("textField_lomchzi3"))))
                        .build();
                List<Map> dataList = (List<Map>) ydClient.queryData(ydParam1, YDConf.FORM_QUERY.retrieve_search_form).getData();
                //如果已存在数据  则不新增
                if (1>dataList.size()){

                    Map hashMap = new HashMap();
                    hashMap.put("associationFormField_lo3pmohz",JSON.parse(map.get("associationFormField_lo3pmohz_id").toString()));  //护理人员名称
                    hashMap.put("textField_lo3pmoi0",map.get("textField_lo3pmoi0"));  //护理人员所属机构
                    hashMap.put("textField_lombve2g",map.get("textField_lomchzi3"));  //护理人员身份证
                    hashMap.put("textField_lo3pmoi3",map.get("textField_lo3pmoi3"));  //护理人员手机号
                    hashMap.put("associationFormField_lo3pmohw",JSON.parse(formData.get("associationFormField_lo3pmohw_id").toString()));  //护理对象姓名
                    hashMap.put("textField_lnyhv5tn",formData.get("textField_lnyhv5tn")); //护理对象身份证
                    hashMap.put("selectField_lnyhv5tp",formData.get("selectField_lnyhv5tp"));  //护理对象等级
                    hashMap.put("textField_lnyhv5tr",formData.get("textField_lnyhv5tr"));  //护理对象住址
                    hashMap.put("selectField_lo3pmohy",dayOfWeek.replace("星期","周"));  //护理时间  （星期几）
                    hashMap.put("dateField_lred6eoj", UtilDateTime.parse(String.valueOf(nextDay)+" "+map.get("selectField_lr1uxpu3"),"yyyy-MM-dd HH:mm")); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                    hashMap.put("dateField_lred6eok",UtilDateTime.parse(String.valueOf(nextDay)+" "+map.get("selectField_lr1uxpu4"),"yyyy-MM-dd HH:mm")); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                    hashMap.put("selectField_lqcgsbw2","未执行");  //执行情况
                    hashMap.put("selectField_lo56u5fn","待上报");  //任务状态

                    hashMap.put("textField_lvae7q5n",formData.get("textField_lvae99fj")); //护理对象经度
                    hashMap.put("textField_lvae7q5o",formData.get("textField_lvae99fk"));  //护理对象纬度

                    hashMap.put("textField_lrx0r38x",String.valueOf(nextDay));  //护理日期-文本
                    hashMap.put("selectField_lr36z6hd","是");  //任务状态
                    hashMap.put("textField_lo55rupj",formData.get("textField_lo55rupj"));  //护理对象姓名-文本
                    hashMap.put("textField_lo3pmoi4",map.get("textField_lo3pmoi4"));  //护理证书类型
                    hashMap.put("employeeField_lo3pmoi6",map.get("employeeField_lo3pmoi6_id"));  //护理人员姓名-成员
                    hashMap.put("textField_lpgi8scj",map.get("textField_lo701pwy"));  //计划明细id
                    hashMap.put("textField_lq9llmhl",formData.get("textField_lop4v4qx"));  //护理对象ID

                    if (formData.get("departmentSelectField_lxztlnnx_id")!=null){
                        hashMap.put("departmentSelectField_lxzto1e1",String.valueOf(formData.get("departmentSelectField_lxztlnnx_id")));  //部门
                    }

                    try {
                        ydClient.operateData(YDParam.builder()
                                .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")
                                .formDataJson(JSON.toJSONString(hashMap))
                                .build(), YDConf.FORM_OPERATION.create);
                    } catch (Exception e) {
                        log.info("定时任务创建计划失败：{}",e);
                    }
                }
            }
        }


    }



    /**
     * 新增护理计划下月月数据
     * @param formInstId
     */
    @Override
    public void creatNextInfo(String formInstId) {
        log.info("新增护理计划下月数据",new Date());
        //根据实例ID  查询护理计划
        YDParam ydParam = YDParam.builder()
                .formInstId(formInstId)
                .build();
        Map formData = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id).getFormData();

        List<Map> mapList = (List<Map>) formData.get("tableField_lo3pmohx");

        LocalDate currentDate = LocalDate.now();
        //获取下月1号的日期
        LocalDate firstDayOfNextMonth  = currentDate.plusMonths(1).with(TemporalAdjusters.firstDayOfMonth());
        //获取下月1号
        int dayOfMonth = firstDayOfNextMonth.getDayOfMonth();
        //获取下月月天数
        int daysInMonth = firstDayOfNextMonth.lengthOfMonth();

        for (int i = 0; i <=daysInMonth-dayOfMonth ; i++) {
            //获取是周几
            String dayOfWeek = firstDayOfNextMonth.plusDays(i).getDayOfWeek().getDisplayName(TextStyle.FULL, new Locale("zh", "CN"));
            //获取循环每天的日期
            LocalDate nextDay = firstDayOfNextMonth.plusDays(i);

            List<Map> maps =  mapList.stream().filter(item -> item.get("selectField_lo3pmohy").equals(dayOfWeek.replace("星期","周"))).collect(Collectors.toList());
            for (Map map : maps) {

                Map hashMap = new HashMap();
                hashMap.put("associationFormField_lo3pmohz",JSON.parse(map.get("associationFormField_lo3pmohz_id").toString()));  //护理人员名称
                hashMap.put("textField_lo3pmoi0",map.get("textField_lo3pmoi0"));  //护理人员所属机构
                hashMap.put("textField_lombve2g",map.get("textField_lomchzi3"));  //护理人员身份证
                hashMap.put("textField_lo3pmoi3",map.get("textField_lo3pmoi3"));  //护理人员手机号
                hashMap.put("associationFormField_lo3pmohw",JSON.parse(formData.get("associationFormField_lo3pmohw_id").toString()));  //护理对象姓名
                hashMap.put("textField_lnyhv5tn",formData.get("textField_lnyhv5tn")); //护理对象身份证
                hashMap.put("selectField_lnyhv5tp",formData.get("selectField_lnyhv5tp"));  //护理对象等级
                hashMap.put("textField_lnyhv5tr",formData.get("textField_lnyhv5tr"));  //护理对象住址
                hashMap.put("selectField_lo3pmohy",dayOfWeek.replace("星期","周"));  //护理时间  （星期几）
                hashMap.put("dateField_lred6eoj", UtilDateTime.parse(String.valueOf(nextDay)+" "+map.get("selectField_lr1uxpu3"),"yyyy-MM-dd HH:mm")); //护理开始时间。 用当前日期+护理计划的  时分    转成时间戳
                hashMap.put("dateField_lred6eok",UtilDateTime.parse(String.valueOf(nextDay)+" "+map.get("selectField_lr1uxpu4"),"yyyy-MM-dd HH:mm")); //护理结束时间。 用当前日期+护理计划的  时分    转成时间戳
                hashMap.put("selectField_lqcgsbw2","未执行");  //执行情况
                hashMap.put("selectField_lo56u5fn","待上报");  //任务状态

                hashMap.put("textField_lvae7q5n",formData.get("textField_lvae99fj")); //护理对象经度
                hashMap.put("textField_lvae7q5o",formData.get("textField_lvae99fk"));  //护理对象纬度

                hashMap.put("textField_lrx0r38x",String.valueOf(nextDay));  //护理日期-文本
                hashMap.put("selectField_lr36z6hd","是");  //任务状态
                hashMap.put("textField_lo55rupj",formData.get("textField_lo55rupj"));  //护理对象姓名-文本
                hashMap.put("textField_lo3pmoi4",map.get("textField_lo3pmoi4"));  //护理证书类型
                hashMap.put("employeeField_lo3pmoi6",map.get("employeeField_lo3pmoi6_id"));  //护理人员姓名-成员
                hashMap.put("textField_lpgi8scj",map.get("textField_lo701pwy"));  //计划明细id
                hashMap.put("textField_lq9llmhl",formData.get("textField_lop4v4qx"));  //护理对象ID

                if (formData.get("departmentSelectField_lxztlnnx_id")!=null){
                    hashMap.put("departmentSelectField_lxzto1e1",String.valueOf(formData.get("departmentSelectField_lxztlnnx_id")));  //部门
                }
                try {
                    ydClient.operateData(YDParam.builder()
                            .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")
                            .formDataJson(JSON.toJSONString(hashMap))
                            .build(), YDConf.FORM_OPERATION.create);
                } catch (Exception e) {
                    log.info("定时任务创建计划失败：{}",e);
                }

            }
        }
    }



    /**
     * 编辑功能只做删除操作
     * @param formInstId
     */
    @SneakyThrows
    @Override
//    @Async
    public void updateInfo(String formInstId) {
        //根据实例ID  查询护理计划
        YDParam ydParam = YDParam.builder()
                .formInstId(formInstId)
                .build();
        Map formData = ydClient.queryData(ydParam, YDConf.FORM_QUERY.retrieve_id).getFormData();

        LocalDate currentDate = LocalDate.now();
        //获取下月1号的日期
        LocalDate firstDayOfNextMonth  = currentDate.plusMonths(1).with(TemporalAdjusters.firstDayOfMonth());
        //删除当前时间节点以后得数据
        //获取需要更新的护理任务  状态为待上报  护理对象身份证   护理日期是当天
        YDParam ydParamDelete = YDParam.builder()
                .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lo56u5fn, textField_lnyhv5tn, dateField_lred6eoj","待上报",formData.get("textField_lnyhv5tn"),Arrays.asList(UtilDateTime.parse( currentDate+ " 00:00:01","yyyy-MM-dd HH:mm:ss"),UtilDateTime.parse( firstDayOfNextMonth.with(TemporalAdjusters.lastDayOfMonth())+ " 23:59:59","yyyy-MM-dd HH:mm:ss")))))
                .build();

        List<Map> queryAllFormData = ydService.queryAllFormData(ydParamDelete);
        queryAllFormData.forEach(item -> {
            log.info("删除",item.get("dateField_lred6eoj"));
            //删除
            ydClient.operateData(YDParam.builder()
                    .formInstanceId(item.get("formInstanceId").toString())
                    .build(), YDConf.FORM_OPERATION.delete);
        });
    }

    /**
     *  删除护理任务
     * @param cId 护理对象身份证
     */
    @Override
    public void deletePlan(String cId) {

        LocalDate currentDate = LocalDate.now();

        //获取下月1号的日期
        LocalDate firstDayOfNextMonth  = currentDate.plusMonths(1).with(TemporalAdjusters.firstDayOfMonth());

        //获取需要删除护理任务  执行状态为未执行  护理对象身份证   护理日期是当天以后
        YDParam ydParamDelete = YDParam.builder()
                .formUuid("FORM-IY966L71PJ8FV10D61M1HBHU6FB320M2765OLM")
                .searchFieldJson(JSON.toJSONString(UtilMap.map("selectField_lqcgsbw2, textField_lnyhv5tn, dateField_lred6eoj","未执行",cId,Arrays.asList(UtilDateTime.parse( currentDate+ " 00:00:01","yyyy-MM-dd HH:mm:ss"),UtilDateTime.parse( firstDayOfNextMonth.with(TemporalAdjusters.lastDayOfMonth())+ " 23:59:59","yyyy-MM-dd HH:mm:ss")))))
                .build();
        List<Map> mapList = ydService.queryAllFormData(ydParamDelete);
        //删除当前时间节点以后得数据
        mapList.forEach(item -> {
            //删除
            ydClient.operateData(YDParam.builder()
                    .formInstanceId(item.get("formInstanceId").toString())
                    .build(), YDConf.FORM_OPERATION.delete);
        });
    }



    /**
     2      * TODO 下载文件到本地
     3      * @author nadim
     4      * @date Sep 11, 2015 11:45:31 AM
     5      * @param fileUrl 远程浏览地址
     6      * @param fileLocal 本地路径
     7      * @throws Exception
     8      */
    public void downloadFile(String fileUrl,String fileLocal) {
        try {
            URL url = new URL(fileUrl);
            HttpURLConnection urlCon = (HttpURLConnection) url.openConnection();
            urlCon.setConnectTimeout(6000);
            urlCon.setReadTimeout(6000);
            int code = urlCon.getResponseCode();
            if (code != HttpURLConnection.HTTP_OK) {
                throw new Exception("文件读取失败");
            }
            //读文件流
            InputStream in = new DataInputStream(urlCon.getInputStream());
            OutputStream out = new DataOutputStream(Files.newOutputStream(Paths.get(fileLocal)));
            byte[] buffer = new byte[2048];
            int count = 0;
            while ((count = in.read(buffer)) > 0) {
                out.write(buffer, 0, count);
            }
            out.close();
            in.close();
        }  catch (IOException e) {
            log.info("读写文件异常");
            e.printStackTrace();
        } catch (Exception e) {
            log.info("处理文件异常");
            e.printStackTrace();
        }
    }
}
